@@ -9,12 +9,13 @@ from django.core.exceptions import ValidationError
 from django.utils import timezone
 from django.utils.translation import ugettext, ugettext_lazy as _
 
-import floppyforms as forms
+import floppyforms.__future__ as forms
 
 from coop_cms.forms.base import InlineHtmlEditableModelForm
 
+from balafon.widgets import CalcHiddenInput
 from balafon.Crm import models
-from balafon.Crm.forms.base import BetterBsModelForm, FormWithFieldsetMixin
+from balafon.Crm.forms.base import BetterBsModelForm, FormWithFieldsetMixin, BsPopupModelForm
 from balafon.Crm.widgets import OpportunityAutoComplete
 
 
@@ -42,47 +43,35 @@ class ActionForm(FormWithFieldsetMixin, BetterBsModelForm):
                     'subject', 'in_charge', 'date', 'time', 'planned_date', 'end_date', 'end_time', 'end_datetime',
                     'opportunity'
                 ],
-                'legend': _(u'Summary')
+                'legend': _('Summary')
             }),
-            ('type', {'fields': ['type', 'status', 'status2', 'amount', 'number'], 'legend': _(u'Type')}),
-            ('details', {'fields': ['detail'], 'legend': _(u'Details')}),
+            ('type', {'fields': ['type', 'status', 'status2', 'amount', 'number'], 'legend': _('Type')}),
+            ('details', {'fields': ['detail'], 'legend': _('Details')}),
         ]
         help_texts = {
-            'amount': _(u'Amount is disabled when value is calculated'),
+            'amount': _('Amount is disabled when value is calculated'),
         }
 
     def __init__(self, *args, **kwargs):
         kwargs.pop('entity', None)
         instance = kwargs.get('instance', None)
+        action_type = kwargs.pop('action_type', None)
         super(ActionForm, self).__init__(*args, **kwargs)
         self.title = u""
-
-        #Force the type to be hidden
-        action_type = None
-        action_type_name = ''
-        is_amount_calculated = False
         if instance:
-            # If a type is already defined and belongs to a set
-            if instance.type and instance.type.set:
-                action_type = instance.type
-            if instance.type:
-                is_amount_calculated = instance.type.is_amount_calculated
-                action_type_name = instance.type.name
-                self.calculated_amount = instance.amount
-        else:
-            # if initial value is provided (from url)
-            action_type = kwargs.get('initial', {}).get('type', None)
-            if action_type:
-                is_amount_calculated = action_type.is_amount_calculated
-                self.calculated_amount = Decimal("0")
-                action_type_name = action_type.name
+            action_type = instance.type
+        self.action_type = action_type
 
+        is_amount_calculated = False
         if action_type:
+            is_amount_calculated = action_type.is_amount_calculated
+            self.calculated_amount = Decimal("0")
             for fieldset_name, fieldset_attrs in self.Meta.fieldsets:
                 if fieldset_name == 'type':
                     fieldset_attrs['legend'] = action_type.name
                     break
             self.fields['type'].widget = forms.HiddenInput()
+            self.fields['type'].initial = action_type
             if instance:
                 self.title = ugettext(u"Edition {0}").format(action_type.name)
             else:
@@ -90,34 +79,56 @@ class ActionForm(FormWithFieldsetMixin, BetterBsModelForm):
         else:
             self.title = ugettext(u"Edit action") if instance else ugettext(u"Create action")
 
+        is_auto_generated = (action_type and action_type.number_auto_generated) or \
+                            (instance and instance.type and instance.type.number_auto_generated)
+        if is_auto_generated:
+            self.fields['number'].widget.attrs['disabled'] = 'disabled'
+            self.fields['number'].required = False
+
         self.fields['amount'].widget.attrs['step'] = 'any'
         if is_amount_calculated:
             self.fields['amount'].widget.attrs['disabled'] = 'disabled'
         self.is_amount_calculated = is_amount_calculated
 
-        if instance and instance.id and instance.type and instance.type.allowed_status.count():
+        if action_type and action_type.allowed_status.count():
             # let javascript disable the blank value if default_status
             choices = [('', "---------")]
+            allowed_status = action_type.allowed_status.all()
+            if instance and instance.frozen:
+                allowed_status = allowed_status.filter(allowed_on_frozen=True)
             self.fields['status'].choices = choices + [
-                (status.id, status.name) for status in instance.type.allowed_status.all()
+                (status.id, status.name) for status in allowed_status
             ]
+            if action_type.default_status:
+                self.fields['status'].initial = action_type.default_status.id
+            else:
+                self.fields['status'].initial = ''
 
-        if instance and instance.id and instance.type and instance.type.allowed_status2.count():
+        if action_type and action_type.allowed_status2.count():
             # let javascript disable the blank value if default_status2
+            allowed_status2 = action_type.allowed_status2.all()
+            if instance and instance.frozen:
+                allowed_status2 = allowed_status2.filter(allowed_on_frozen=True)
             choices = [('', "---------")]
             self.fields['status2'].choices = choices + [
-                (status.id, status.name) for status in instance.type.allowed_status2.all()
+                (status.id, status.name) for status in allowed_status2
             ]
+            if action_type.default_status2:
+                self.fields['status2'].initial = action_type.default_status2.id
+            else:
+                self.fields['status2'].initial = ''
+        else:
+            self.fields['status2'].widget = forms.HiddenInput()
 
         self.fields['opportunity'].widget = forms.HiddenInput()
-        self.fields['detail'].widget = forms.Textarea(attrs={'placeholder': _(u'enter details'), 'cols': '72'})
+        self.fields['detail'].widget = forms.Textarea(attrs={'placeholder': _('enter details'), 'cols': '72'})
 
         self._init_dt_field("planned_date", "date", "time")
         self._init_dt_field("end_datetime", "end_date", "end_time")
 
     def _init_dt_field(self, dt_field, date_field, time_field):
         """init datetime fields"""
-        self.fields[dt_field].widget = forms.HiddenInput()
+        self.fields[dt_field].widget = CalcHiddenInput()
         the_datetime = getattr(self.instance, dt_field) if self.instance else self.fields[dt_field].initial
         if the_datetime:
             self.fields[date_field].initial = the_datetime.date()
@@ -128,9 +139,14 @@ class ActionForm(FormWithFieldsetMixin, BetterBsModelForm):
             else:
                 self.fields[time_field].initial = the_datetime.time()
 
+        is_frozen = self.instance.frozen if self.instance else False
+        if is_frozen:
+            self.fields[date_field].widget.attrs['disabled'] = 'disabled'
+            self.fields[time_field].widget.attrs['disabled'] = 'disabled'
+
     def clean_status(self):
         """status validation"""
-        type_of = self.cleaned_data['type']
+        type_of = self.cleaned_data.get('type') or self.action_type
         status = self.cleaned_data['status']
         if type_of:
             allowed_status = ([] if type_of.default_status else [None]) + list(type_of.allowed_status.all())
@@ -171,7 +187,7 @@ class ActionForm(FormWithFieldsetMixin, BetterBsModelForm):
         return the_time
 
     def clean_end_date(self):
-        """end date valodation"""
+        """end date validation"""
         date1 = self.cleaned_data.get("date", None)
         date2 = self.cleaned_data.get("end_date", None)
         if date2:
@@ -266,12 +282,12 @@ class OpportunityForm(FormWithFieldsetMixin, BetterBsModelForm):
         fields = ('name', 'detail')
 
         fieldsets = [
-            ('name', {'fields': ['name', 'detail'], 'legend': _(u'Summary')}),
+            ('name', {'fields': ['name', 'detail'], 'legend': _('Summary')}),
         ]
 
     def __init__(self, *args, **kwargs):
         super(OpportunityForm, self).__init__(*args, **kwargs)
-        self.fields['detail'].widget = forms.Textarea(attrs={'placeholder': _(u'enter details'), 'cols':'72'})
+        self.fields['detail'].widget = forms.Textarea(attrs={'placeholder': _('enter details'), 'cols':'72'})
 
 
 class OpportunityStatusForm(forms.ModelForm):
@@ -290,7 +306,7 @@ class SelectOpportunityForm(forms.Form):
         kwargs.pop('choices', None)
         super(SelectOpportunityForm, self).__init__(*args, **kwargs)
         widget = OpportunityAutoComplete(
-            attrs={'placeholder': _(u'Enter the name of a opportunity'), 'size': '50', 'class': 'colorbox'})
+            attrs={'placeholder': _('Enter the name of a opportunity'), 'size': '50', 'class': 'colorbox'})
         self.fields["opportunity"] = forms.CharField(label=_(u"Opportunity"), widget=widget)
 
     def clean_opportunity(self):
@@ -304,7 +320,7 @@ class SelectOpportunityForm(forms.Form):
 
 class CloneActionForm(forms.Form):
     """form for clone_action: choose which type to chooses"""
-    action_type = forms.ChoiceField(required=True, choices=[], label=_(u'New type'))
+    action_type = forms.ChoiceField(required=True, choices=[], label=_('New type'))
 
     def __init__(self, action_type, *args, **kwargs):
         super(CloneActionForm, self).__init__(*args, **kwargs)
@@ -314,7 +330,7 @@ class CloneActionForm(forms.Form):
             for action_type in action_type.next_action_types.all().order_by('order_index')
         ]
         self.fields['action_type'].choices = choices
-        #If only 1 choice = change it to a confirmation
+        # If only 1 choice = change it to a confirmation
         if len(choices) == 1:
             self.fields['action_type'].initial = choices[0][0]
             self.fields['action_type'].widget = forms.HiddenInput()
@@ -329,17 +345,17 @@ class CloneActionForm(forms.Form):
         try:
             return models.ActionType.objects.get(id=action_type)
         except models.ActionType.DoesNotExist:
-            raise forms.ValidationError(_(u"Invalid type"))
+            raise forms.ValidationError(_("Invalid type"))
 
 
-class UpdateActionStatusForm(forms.ModelForm):
+class UpdateActionStatusForm(BsPopupModelForm):
     """form changing the status"""
 
     class Meta:
         """form from model"""
         model = models.Action
         fields = (
-            'status',
+            'status', 'status2',
         )
 
     def __init__(self, *args, **kwargs):
@@ -348,21 +364,44 @@ class UpdateActionStatusForm(forms.ModelForm):
 
         if instance and instance.id and instance.type and instance.type.allowed_status.count():
             # let javascript disable the blank value if default_status
+            allowed_status = instance.type.allowed_status.all()
+            if instance.frozen:
+                allowed_status = allowed_status.filter(allowed_on_frozen=True)
             self.fields['status'].choices = [
-                (status.id, status.name) for status in instance.type.allowed_status.all()
+                (status.id, status.name) for status in allowed_status
             ]
 
+        if instance and instance.id and instance.type and instance.type.allowed_status2.count():
+            # let javascript disable the blank value if default_status2
+            allowed_status2 = instance.type.allowed_status2.all()
+            if instance.frozen:
+                allowed_status2 = allowed_status2.filter(allowed_on_frozen=True)
+            self.fields['status2'].choices = [
+                (status.id, status.name) for status in allowed_status2
+            ]
+        else:
+            self.fields['status2'].widget = forms.HiddenInput()
+
     def clean_status(self):
         """status validation"""
         status = self.cleaned_data['status']
         action_type = self.instance.type
         allowed_status = ([] if action_type.default_status else [None]) + list(action_type.allowed_status.all())
         if len(allowed_status) > 0 and status not in allowed_status:
-            raise ValidationError(ugettext(u"This status can't not be used for this action type"))
+            raise ValidationError(ugettext("This status can't not be used for this action type"))
+        return status
+
+    def clean_status2(self):
+        """status validation"""
+        status = self.cleaned_data['status2']
+        action_type = self.instance.type
+        allowed_status = ([] if action_type.default_status2 else [None]) + list(action_type.allowed_status2.all())
+        if len(allowed_status) > 0 and status not in allowed_status:
+            raise ValidationError(ugettext("This status can't not be used for this action type"))
         return status
 
 
-class UpdateActionStatus2Form(forms.ModelForm):
+class UpdateActionStatus2Form(BsPopupModelForm):
     """form changing the status"""
 
     class Meta:
@@ -388,7 +427,7 @@ class UpdateActionStatus2Form(forms.ModelForm):
         action_type = self.instance.type
         allowed_status = ([] if action_type.default_status2 else [None]) + list(action_type.allowed_status2.all())
         if len(allowed_status) > 0 and status not in allowed_status:
-            raise ValidationError(ugettext(u"This status can't not be used for this action type"))
+            raise ValidationError(ugettext("This status can't not be used for this action type"))
         return status
 
 
