@@ -7,11 +7,13 @@ from django.db.models import Q, Count
 from django.contrib.auth.models import User
 from django.utils.translation import ugettext as _
 
-import floppyforms as forms
+import floppyforms.__future__ as forms
 
 from balafon.Crm import models
 from balafon.Crm.settings import get_language_choices
-from balafon.Crm.utils import get_default_country
+from balafon.Crm.utils import (
+    get_default_country, sort_by_name_callback, sort_by_entity_callback, sort_by_contact_callback
+)
 from balafon.Crm.widgets import CityNoCountryAutoComplete, GroupAutoComplete
 from balafon.Search.forms import SearchFieldForm, TwoDatesForm, YesNoSearchFieldForm
 
@@ -19,13 +21,13 @@ from balafon.Search.forms import SearchFieldForm, TwoDatesForm, YesNoSearchField
 class EntityNameSearchForm(SearchFieldForm):
     """by entity name"""
     name = 'entity_name'
-    label = _(u'Entity name')
+    label = _('Entity name')
     
     def __init__(self, *args, **kwargs):
         super(EntityNameSearchForm, self).__init__(*args, **kwargs)
         field = forms.CharField(
             label=self.label,
-            widget=forms.TextInput(attrs={'placeholder': _(u'Enter a part of the name of the searched entities')})
+            widget=forms.TextInput(attrs={'placeholder': _('Enter a part of the name of the searched entities')})
         )
         self._add_field(field)
         
@@ -38,14 +40,14 @@ class EntityDescriptionForm(SearchFieldForm):
     """by entity description"""
 
     name = 'entity_description'
-    label = _(u'Entity description')
+    label = _('Entity description')
     
     def __init__(self, *args, **kwargs):
         super(EntityDescriptionForm, self).__init__(*args, **kwargs)
         field = forms.CharField(
             label=self.label,
             widget=forms.TextInput(
-                attrs={'placeholder': _(u'Enter a part of the description of the searched entities')}
+                attrs={'placeholder': _('Enter a part of the description of the searched entities')}
             )
         )
         self._add_field(field)
@@ -59,13 +61,13 @@ class EntityNotesForm(SearchFieldForm):
     """by entity notes"""
 
     name = 'entity_notes'
-    label = _(u'Entity notes')
+    label = _('Entity notes')
     
     def __init__(self, *args, **kwargs):
         super(EntityNotesForm, self).__init__(*args, **kwargs)
         field = forms.CharField(
             label=self.label,
-            widget=forms.TextInput(attrs={'placeholder': _(u'Enter a part of the notes of the searched entities')})
+            widget=forms.TextInput(attrs={'placeholder': _('Enter a part of the notes of the searched entities')})
         )
         self._add_field(field)
         
@@ -78,14 +80,14 @@ class EntityNameStartsWithSearchForm(SearchFieldForm):
     """by entity name starts with"""
 
     name = 'entity_name_sw'
-    label = _(u'Entity name starts with')
+    label = _('Entity name starts with')
     
     def __init__(self, *args, **kwargs):
         super(EntityNameStartsWithSearchForm, self).__init__(*args, **kwargs)
         field = forms.CharField(
             label=self.label,
             widget=forms.TextInput(
-                attrs={'placeholder': _(u'Enter the beginning of the name of the searched entities')}
+                attrs={'placeholder': _('Enter the beginning of the name of the searched entities')}
             )
         )
         self._add_field(field)
@@ -99,14 +101,14 @@ class AddressSearchForm(SearchFieldForm):
     """by address contains"""
 
     name = 'address'
-    label = _(u'Contact or entity at this address')
+    label = _('Contact or entity at this address')
 
     def __init__(self, *args, **kwargs):
         super(AddressSearchForm, self).__init__(*args, **kwargs)
         field = forms.CharField(
             label=self.label,
             widget=forms.TextInput(
-                attrs={'placeholder': _(u'Enter text contained in an address')}
+                attrs={'placeholder': _('Enter text contained in an address')}
             )
         )
         self._add_field(field)
@@ -125,7 +127,7 @@ class HasEntityForm(YesNoSearchFieldForm):
     """contacts member of an entity"""
 
     name = 'has_entity'
-    label = _(u'Has entity?')
+    label = _('Has entity?')
         
     def get_lookup(self):
         """lookup"""
@@ -144,7 +146,7 @@ class BaseCitySearchForm(SearchFieldForm):
         field = forms.ModelChoiceField(
             queryset,
             label=self.label,
-            widget=CityNoCountryAutoComplete(attrs={'placeholder': _(u'Enter a city'), 'size': '80'})
+            widget=CityNoCountryAutoComplete(attrs={'placeholder': _('Enter a city'), 'size': '80'})
         )
         self._add_field(field)
 
@@ -152,7 +154,7 @@ class BaseCitySearchForm(SearchFieldForm):
 class CitySearchForm(BaseCitySearchForm):
     """by city"""
     name = 'city'
-    label = _(u'City')
+    label = _('City')
     
     def get_lookup(self):
         """lookup"""
@@ -162,7 +164,7 @@ class CitySearchForm(BaseCitySearchForm):
 class EntityCitySearchForm(BaseCitySearchForm):
     """by city of the entity"""
     name = 'entity_city'
-    label = _(u'Entity city')
+    label = _('Entity city')
 
     def get_lookup(self):
         """lookup"""
@@ -175,7 +177,7 @@ class BaseZipCodeSearchForm(SearchFieldForm):
         super(BaseZipCodeSearchForm, self).__init__(*args, **kwargs)
         field = forms.CharField(
             label=self.label,
-            widget=forms.TextInput(attrs={'placeholder': _(u'Enter the beginning of the zip code')})
+            widget=forms.TextInput(attrs={'placeholder': _('Enter the beginning of the zip code')})
         )
         self._add_field(field)
     
@@ -183,7 +185,7 @@ class BaseZipCodeSearchForm(SearchFieldForm):
 class ZipCodeSearchForm(BaseZipCodeSearchForm):
     """by zip code"""
     name = 'zip_code'
-    label = _(u'zip code')
+    label = _('zip code')
     
     def get_lookup(self):
         """lookup"""
@@ -193,7 +195,7 @@ class ZipCodeSearchForm(BaseZipCodeSearchForm):
 class EntityZipCodeSearchForm(BaseZipCodeSearchForm):
     """by zip code of the entity"""
     name = 'entity_zip_code'
-    label = _(u'Entity zip code')
+    label = _('Entity zip code')
             
     def get_lookup(self):
         """lookup"""
@@ -214,7 +216,7 @@ class ZoneSearchForm(SearchFieldForm):
         if widget:
             kwargs['widget'] = widget
         field = forms.MultipleChoiceField(
-            choices=[(x.id, unicode(x)) for x in queryset.all()],
+            choices=[(x.id, '{0}'.format(x)) for x in queryset.all()],
             label=self.label,
             **kwargs
         )
@@ -233,7 +235,8 @@ class ZoneSearchForm(SearchFieldForm):
         return forms.SelectMultiple(attrs={
             'class': "chosen-select",
             'data-placeholder': self.label,
-            'style': "width: 100%", 
+            'style': "width: 100%",
+            'width': '100%',
         })
     
     def get_queryset(self, queryset):
@@ -248,7 +251,7 @@ class ZoneSearchForm(SearchFieldForm):
 class DepartmentSearchForm(ZoneSearchForm):
     """by departement"""
     name = 'department'
-    label = _(u'Departments')
+    label = _('Departments')
         
     def get_lookup(self):
         """lookup"""
@@ -268,7 +271,7 @@ class DepartmentSearchForm(ZoneSearchForm):
 class EntityDepartmentSearchForm(ZoneSearchForm):
     """by departement of the entity"""
     name = 'entity_department'
-    label = _(u'Entity Departments')
+    label = _('Entity Departments')
         
     def get_queryset(self, queryset):
         """queryset"""
@@ -283,7 +286,7 @@ class EntityDepartmentSearchForm(ZoneSearchForm):
 class RegionSearchForm(ZoneSearchForm):
     """by region"""
     name = 'region'
-    label = _(u'Regions')
+    label = _('Regions')
         
     def get_lookup(self):
         """lookup"""
@@ -303,7 +306,7 @@ class RegionSearchForm(ZoneSearchForm):
 class EntityRegionSearchForm(ZoneSearchForm):
     """by region of teh entity"""
     name = 'entity_region'
-    label = _(u'Entity Regions')
+    label = _('Entity Regions')
     
     def get_queryset(self, queryset):
         """queryset"""
@@ -318,7 +321,7 @@ class EntityRegionSearchForm(ZoneSearchForm):
 class LargeRegionSearchForm(ZoneSearchForm):
     """by region"""
     name = 'large_region'
-    label = _(u'Large Regions')
+    label = _('Large Regions')
 
     def get_lookup(self):
         """lookup"""
@@ -336,7 +339,7 @@ class LargeRegionSearchForm(ZoneSearchForm):
 class EntityLargeRegionSearchForm(ZoneSearchForm):
     """by region of teh entity"""
     name = 'entity_large_region'
-    label = _(u'Entity Large Regions')
+    label = _('Entity Large Regions')
 
     def get_lookup(self):
         """queryset"""
@@ -346,7 +349,7 @@ class EntityLargeRegionSearchForm(ZoneSearchForm):
 class CountrySearchForm(ZoneSearchForm):
     """by country"""
     name = 'country'
-    label = _(u'Countries')
+    label = _('Countries')
         
     def get_lookup(self):
         """lookup"""
@@ -364,7 +367,7 @@ class CountrySearchForm(ZoneSearchForm):
 class EntityCountrySearchForm(ZoneSearchForm):
     """by country of the entity"""
     name = 'entity_country'
-    label = _(u'Entity Countries')
+    label = _('Entity Countries')
         
     def get_lookup(self):
         """lookup"""
@@ -379,7 +382,7 @@ class EntityCountrySearchForm(ZoneSearchForm):
 class ZoneGroupSearchForm(ZoneSearchForm):
     """by zone group"""
     name = 'zone_group'
-    label = _(u'Zone Groups')
+    label = _('Zone Groups')
         
     def get_lookup(self):
         """lookup"""
@@ -391,7 +394,7 @@ class ZoneGroupSearchForm(ZoneSearchForm):
 class EntityZoneGroupSearchForm(ZoneSearchForm):
     """by zone group of entity"""
     name = 'entity_zone_group'
-    label = _(u'Entity Zone Groups')
+    label = _('Entity Zone Groups')
         
     def get_lookup(self):
         """lookup"""
@@ -401,7 +404,7 @@ class EntityZoneGroupSearchForm(ZoneSearchForm):
 class HasCityAndZipcodeForm(YesNoSearchFieldForm):
     """by has city and address"""
     name = 'has_city_and_zip'
-    label = _(u'Has city and zip code?')
+    label = _('Has city and zip code?')
         
     def get_lookup(self):
         """lookup"""
@@ -417,7 +420,7 @@ class HasCityAndZipcodeForm(YesNoSearchFieldForm):
 class HasAddresseForm(YesNoSearchFieldForm):
     """by has city and address"""
     name = 'has_address'
-    label = _(u'Has address?')
+    label = _('Has address?')
 
     def get_lookup(self):
         """lookup"""
@@ -433,7 +436,7 @@ class HasAddresseForm(YesNoSearchFieldForm):
 class ActionInProgressForm(YesNoSearchFieldForm):
     """by action in progress"""
     name = 'action'
-    label = _(u'Action in progress')
+    label = _('Action in progress')
     is_action_form = True
 
     def get_queryset(self, queryset):
@@ -448,7 +451,7 @@ class ActionInProgressForm(YesNoSearchFieldForm):
 class HasAction(YesNoSearchFieldForm):
     """Has an action"""
     name = 'has_action'
-    label = _(u'Has actions')
+    label = _('Has actions')
         
     def get_queryset(self, queryset):
         """queryset"""
@@ -462,7 +465,7 @@ class HasAction(YesNoSearchFieldForm):
 class ActionByDoneDate(TwoDatesForm):
     """by action done between two dates"""
     name = 'action_by_done_date'
-    label = _(u'Action by done date')
+    label = _('Action by done date')
     is_action_form = True
         
     def get_lookup(self):
@@ -474,7 +477,7 @@ class ActionByDoneDate(TwoDatesForm):
 class ActionByPlannedDate(TwoDatesForm):
     """by action planned between two dates"""
     name = 'action_by_planned_date'
-    label = _(u'Action by planned date')
+    label = _('Action by planned date')
     is_action_form = True
     
     def get_lookup(self):
@@ -503,7 +506,7 @@ class ActionByPlannedDate(TwoDatesForm):
 class ActionByStartDate(TwoDatesForm):
     """By action started between two dates"""
     name = 'action_by_start_date'
-    label = _(u'Action by start date')
+    label = _('Action by start date')
     is_action_form = True
     
     def get_lookup(self):
@@ -515,7 +518,7 @@ class ActionByStartDate(TwoDatesForm):
 class ActionByUser(SearchFieldForm):
     """by user in charge of an action"""
     name = 'action_by_user'
-    label = _(u'Action by user')
+    label = _('Action by user')
     is_action_form = True
     
     def __init__(self, *args, **kwargs):
@@ -532,7 +535,7 @@ class ActionByUser(SearchFieldForm):
 class ActionGteAmount(SearchFieldForm):
     """by action with amount greater than a value"""
     name = 'action_gte_amount'
-    label = _(u'Action with amount greater or equal to')
+    label = _('Action with amount greater or equal to')
     is_action_form = True
     
     def __init__(self, *args, **kwargs):
@@ -548,7 +551,7 @@ class ActionGteAmount(SearchFieldForm):
 class ActionLtAmount(SearchFieldForm):
     """by action with amount less than a value"""
     name = 'action_lt_amount'
-    label = _(u'Action with amount less than')
+    label = _('Action with amount less than')
     is_action_form = True
     
     def __init__(self, *args, **kwargs):
@@ -564,7 +567,7 @@ class ActionLtAmount(SearchFieldForm):
 class ActionStatus(SearchFieldForm):
     """by action status"""
     name = 'action_status'
-    label = _(u'Contacts with action of status')
+    label = _('Contacts with action of status')
     is_action_form = True
     
     def __init__(self, *args, **kwargs):
@@ -581,7 +584,7 @@ class ActionStatus(SearchFieldForm):
 class ActionWithoutStatus(SearchFieldForm):
     """by action status"""
     name = 'action_without_status'
-    label = _(u'Contacts without action of status')
+    label = _('Contacts without action of status')
     is_action_form = True
 
     def __init__(self, *args, **kwargs):
@@ -598,7 +601,7 @@ class ActionWithoutStatus(SearchFieldForm):
 class TypeSearchForm(SearchFieldForm):
     """by entity type"""
     name = 'type'
-    label = _(u'Entity type')
+    label = _('Entity type')
     
     def __init__(self, *args, **kwargs):
         super(TypeSearchForm, self).__init__(*args, **kwargs)
@@ -614,12 +617,12 @@ class TypeSearchForm(SearchFieldForm):
 class GroupSearchForm(SearchFieldForm):
     """by group"""
     name = 'group'
-    label = _(u'Group')
+    label = _('Group')
     
     def _get_widget(self):
         """customize widget: autocomplete"""
         return GroupAutoComplete(attrs={
-            'placeholder': _(u'Enter part of the group name'), 'size': '80',
+            'placeholder': _('Enter part of the group name'), 'size': '80',
         })
 
     def __init__(self, *args, **kwargs):
@@ -627,7 +630,7 @@ class GroupSearchForm(SearchFieldForm):
         queryset = models.Group.objects.all()
         try:
             queryset = queryset.extra(select={'lower_name': 'lower(name)'}).order_by('lower_name')
-        except Exception: #pylint: disable=broad-except
+        except Exception:  # pylint: disable=broad-except
             queryset = queryset.order_by('name')
         kwargs = {}
         widget = self._get_widget()
@@ -644,13 +647,13 @@ class GroupSearchForm(SearchFieldForm):
 class GroupSearchFormDropdownWidget(GroupSearchForm):
     """Search by group: dropdown widget"""
     name = 'group_dropdown'
-    label = _(u'Group (dropdown list)')
+    label = _('Group (dropdown list)')
     
     def _get_widget(self):
         """dropdown widget"""
         return forms.Select(attrs={
             'class': "chosen-select",
-            'data-placeholder': _(u'Group names'),
+            'data-placeholder': _('Group names'),
             'style': "width: 100%", 
         })
 
@@ -672,7 +675,7 @@ class MultiGroupSearchForm(SearchFieldForm):
         if widget:
             kwargs['widget'] = widget
         field = forms.MultipleChoiceField(
-            choices=[(x.id, unicode(x)) for x in queryset.all()], label=self.label, **kwargs)
+            choices=[(x.id, '{0}'.format(x)) for x in queryset.all()], label=self.label, **kwargs)
         self._add_field(field)
         
     def get_values(self):
@@ -687,15 +690,16 @@ class MultiGroupSearchForm(SearchFieldForm):
         """customize widget"""
         return forms.SelectMultiple(attrs={
             'class': "chosen-select",
-            'data-placeholder': _(u'Select Group names'),
-            'style': "width: 100%", 
+            'data-placeholder': _('Select Group names'),
+            'style': "width: 100%",
+            'width': "100%",
         })
 
 
 class GroupsMemberOfAllSearchForm(MultiGroupSearchForm):
     """members of all groups"""
     name = 'all_groups'
-    label = _(u'Members of all groups')
+    label = _('Members of all groups')
     
     def get_queryset(self, queryset):
         """queryset"""
@@ -708,7 +712,7 @@ class GroupsMemberOfAllSearchForm(MultiGroupSearchForm):
 class GroupsMemberOfAnySearchForm(MultiGroupSearchForm):
     """member of one of the groups"""
     name = 'any_groups'
-    label = _(u'Members of at least one group')
+    label = _('Members of at least one group')
     multi_values = True
     
     def get_queryset(self, queryset):
@@ -724,7 +728,7 @@ class GroupsMemberOfAnySearchForm(MultiGroupSearchForm):
 class GroupsMemberOfNoneSearchForm(MultiGroupSearchForm):
     """not member of any of the groups"""
     name = 'none_groups'
-    label = _(u'Member of none of these groups')
+    label = _('Member of none of these groups')
     multi_values = True
     
     def get_queryset(self, queryset):
@@ -738,19 +742,19 @@ class GroupsMemberOfNoneSearchForm(MultiGroupSearchForm):
 class NotInGroupSearchForm(SearchFieldForm):
     """not in group"""
     name = 'not_in_group'
-    label = _(u'Not in group')
+    label = _('Not in group')
     
     def __init__(self, *args, **kwargs):
         super(NotInGroupSearchForm, self).__init__(*args, **kwargs)
         queryset = models.Group.objects.all()
         try:
             queryset = queryset.extra(select={'lower_name': 'lower(name)'}).order_by('lower_name')
-        except Exception: #pylint: disable=broad-except
+        except Exception:  # pylint: disable=broad-except
             queryset = queryset.order_by('name')
         field = forms.ModelChoiceField(
             queryset,
             label=self.label,
-            widget=GroupAutoComplete(attrs={'placeholder': _(u'Enter part of the group name'), 'size': '80'})
+            widget=GroupAutoComplete(attrs={'placeholder': _('Enter part of the group name'), 'size': '80'})
         )
         self._add_field(field)
     
@@ -762,7 +766,7 @@ class NotInGroupSearchForm(SearchFieldForm):
 class ContactAgeSearchForm(SearchFieldForm):
     """search by age"""
     name = 'contact_age'
-    label = _(u'Contact age')
+    label = _('Contact age')
     
     def __init__(self, *args, **kwargs):
         super(ContactAgeSearchForm, self).__init__(*args, **kwargs)
@@ -780,7 +784,7 @@ class ContactAgeSearchForm(SearchFieldForm):
 class ContactAcceptSubscriptionSearchForm(SearchFieldForm):
     """by accept subscrition"""
     name = 'accept_subscription'
-    label = _(u'Accept subscription to')
+    label = _('Accept subscription to')
     
     def __init__(self, *args, **kwargs):
         super(ContactAcceptSubscriptionSearchForm, self).__init__(*args, **kwargs)
@@ -796,7 +800,7 @@ class ContactAcceptSubscriptionSearchForm(SearchFieldForm):
 class ContactRefuseSubscriptionSearchForm(ContactAcceptSubscriptionSearchForm):
     """by refuse subscription"""
     name = 'refuse_subscription'
-    label = _(u'Refuse subscription to')
+    label = _('Refuse subscription to')
     
     def get_lookup(self):
         """lookup"""
@@ -810,7 +814,7 @@ class ContactRefuseSubscriptionSearchForm(ContactAcceptSubscriptionSearchForm):
 class SecondarySearchForm(SearchFieldForm):
     """by secondary contact"""
     name = 'secondary_contact'
-    label = _(u'Secondary contact')
+    label = _('Secondary contact')
     
     def __init__(self, *args, **kwargs):
         super(SecondarySearchForm, self).__init__(*args, **kwargs)
@@ -831,7 +835,7 @@ class SecondarySearchForm(SearchFieldForm):
 class ContactHasLeft(SearchFieldForm):
     """contact who left"""
     name = 'contact_has_left'
-    label = _(u'Contact has left')
+    label = _('Contact has left')
     
     def __init__(self, *args, **kwargs):
         super(ContactHasLeft, self).__init__(*args, **kwargs)
@@ -852,7 +856,7 @@ class ContactHasLeft(SearchFieldForm):
 class ContactRoleSearchForm(SearchFieldForm):
     """by role"""
     name = 'contact_role'
-    label = _(u'Contact role')
+    label = _('Contact role')
     
     def __init__(self, *args, **kwargs):
         super(ContactRoleSearchForm, self).__init__(*args, **kwargs)
@@ -868,13 +872,13 @@ class ContactRoleSearchForm(SearchFieldForm):
 class EmailSearchForm(SearchFieldForm):
     """by email"""
     name = 'contact_entity_email'
-    label = _(u'Email')
+    label = _('Email')
     
     def __init__(self, *args, **kwargs):
         super(EmailSearchForm, self).__init__(*args, **kwargs)
         field = forms.CharField(
             label=self.label,
-            widget=forms.TextInput(attrs={'placeholder': _(u'Enter a part of the email of a contact or an entity')})
+            widget=forms.TextInput(attrs={'placeholder': _('Enter a part of the email of a contact or an entity')})
         )
         self._add_field(field)
         
@@ -886,7 +890,7 @@ class EmailSearchForm(SearchFieldForm):
 class ContactHasEmail(YesNoSearchFieldForm):
     """by has email"""
     name = 'contact_has_email'
-    label = _(u'Contact has email')
+    label = _('Contact has email')
             
     def get_lookup(self):
         """lookup"""
@@ -900,7 +904,7 @@ class ContactHasEmail(YesNoSearchFieldForm):
 class ContactHasPersonalEmail(YesNoSearchFieldForm):
     """by has an email set on the contact (ignore if set on entity)"""
     name = 'contact_has_personal_email'
-    label = _(u'Contact has pesonal email')
+    label = _('Contact has pesonal email')
             
     def get_lookup(self):
         """queryset"""
@@ -914,7 +918,7 @@ class ContactHasPersonalEmail(YesNoSearchFieldForm):
 class UnknownContact(YesNoSearchFieldForm):
     """no name"""
     name = 'unknown_contact'
-    label = _(u'Unknown contacts')
+    label = _('Unknown contacts')
         
     def get_lookup(self):
         """lookup"""
@@ -928,7 +932,7 @@ class UnknownContact(YesNoSearchFieldForm):
 class ActionTypeSearchForm(SearchFieldForm):
     """by type of action"""
     name = 'action_type'
-    label = _(u'Action type')
+    label = _('Action type')
     is_action_form = True
 
     def __init__(self, *args, **kwargs):
@@ -945,14 +949,14 @@ class ActionTypeSearchForm(SearchFieldForm):
 class ActionNameSearchForm(SearchFieldForm):
     """by subject of action"""
     name = 'action_name'
-    label = _(u'Action subject')
+    label = _('Action subject')
     is_action_form = True
 
     def __init__(self, *args, **kwargs):
         super(ActionNameSearchForm, self).__init__(*args, **kwargs)
         field = forms.CharField(
             label=self.label,
-            widget=forms.TextInput(attrs={'placeholder': _(u'enter a part of the name of the searched action')})
+            widget=forms.TextInput(attrs={'placeholder': _('enter a part of the name of the searched action')})
         )
         self._add_field(field)
         
@@ -961,10 +965,18 @@ class ActionNameSearchForm(SearchFieldForm):
         return Q(subject__icontains=self.value)
 
 
+class ExcludeActionNameSearchForm(ActionNameSearchForm):
+    """by subject of action"""
+    name = 'exclude_action_name'
+    label = _(u'No Action with subject')
+    is_action_form = True
+    is_exclude_action_form = True
+
+
 class RelationshipDateForm(TwoDatesForm):
     """by date of relationship"""
     name = 'relationship_date'
-    label = _(u'Relationship date')
+    label = _('Relationship date')
             
     def get_lookup(self):
         """lookup"""
@@ -975,13 +987,13 @@ class RelationshipDateForm(TwoDatesForm):
 class ContactNameSearchForm(SearchFieldForm):
     """by contact name"""
     name = 'contact_name'
-    label = _(u'Contact name')
+    label = _('Contact name')
     
     def __init__(self, *args, **kwargs):
         super(ContactNameSearchForm, self).__init__(*args, **kwargs)
         field = forms.CharField(
             label=self.label,
-            widget=forms.TextInput(attrs={'placeholder': _(u'Enter a part of the name of the searched contact')})
+            widget=forms.TextInput(attrs={'placeholder': _('Enter a part of the name of the searched contact')})
         )
         self._add_field(field)
         
@@ -993,7 +1005,7 @@ class ContactNameSearchForm(SearchFieldForm):
 class ContactLanguageSearchForm(SearchFieldForm):
     """by contact language"""
     name = 'contact_lang'
-    label = _(u'Contact language')
+    label = _('Contact language')
 
     def __init__(self, *args, **kwargs):
         super(ContactLanguageSearchForm, self).__init__(*args, **kwargs)
@@ -1012,13 +1024,13 @@ class ContactLanguageSearchForm(SearchFieldForm):
 class ContactFirstnameSearchForm(SearchFieldForm):
     """by firstname"""
     name = 'contact_firstname'
-    label = _(u'Contact firstname')
+    label = _('Contact firstname')
     
     def __init__(self, *args, **kwargs):
         super(ContactFirstnameSearchForm, self).__init__(*args, **kwargs)
         field = forms.CharField(
             label=self.label,
-            widget=forms.TextInput(attrs={'placeholder': _(u'Enter a part of the firstname of the searched contact')})
+            widget=forms.TextInput(attrs={'placeholder': _('Enter a part of the firstname of the searched contact')})
         )
         self._add_field(field)
         
@@ -1030,13 +1042,13 @@ class ContactFirstnameSearchForm(SearchFieldForm):
 class ContactNotesSearchForm(SearchFieldForm):
     """by notes"""
     name = 'contact_notes'
-    label = _(u'Contact notes')
+    label = _('Contact notes')
     
     def __init__(self, *args, **kwargs):
         super(ContactNotesSearchForm, self).__init__(*args, **kwargs)
         field = forms.CharField(
             label=self.label,
-            widget=forms.TextInput(attrs={'placeholder': _(u'Enter a part of a note of the searched contact')})
+            widget=forms.TextInput(attrs={'placeholder': _('Enter a part of a note of the searched contact')})
         )
         self._add_field(field)
         
@@ -1048,7 +1060,7 @@ class ContactNotesSearchForm(SearchFieldForm):
 class OpportunitySearchForm(SearchFieldForm):
     """by opportunity"""
     name = 'opportunity'
-    label = _(u'Opportunity')
+    label = _('Opportunity')
     is_action_form = True
     
     def __init__(self, *args, **kwargs):
@@ -1065,14 +1077,14 @@ class OpportunitySearchForm(SearchFieldForm):
 class OpportunityNameSearchForm(SearchFieldForm):
     """by opportunity name"""
     name = 'opportunity_name'
-    label = _(u'Opportunity name')
+    label = _('Opportunity name')
     is_action_form = True
     
     def __init__(self, *args, **kwargs):
         super(OpportunityNameSearchForm, self).__init__(*args, **kwargs)
         field = forms.CharField(
             label=self.label,
-            widget=forms.TextInput(attrs={'placeholder': _(u'enter a part of the name of the searched opportunity')})
+            widget=forms.TextInput(attrs={'placeholder': _('enter a part of the name of the searched opportunity')})
         )
         self._add_field(field)
         
@@ -1084,7 +1096,7 @@ class OpportunityNameSearchForm(SearchFieldForm):
 class NoSameAsForm(YesNoSearchFieldForm):
     """Allow same as contact in results"""
     name = 'no_same_as'
-    label = _(u'Allow same-as')
+    label = _('Allow same-as')
     
     def get_lookup(self):
         """lookup"""
@@ -1121,7 +1133,7 @@ class NoSameAsForm(YesNoSearchFieldForm):
 class HasSameAsForm(SearchFieldForm):
     """all contacts with same as"""
     name = 'has_same_as'
-    label = _(u'Has same-as')
+    label = _('Has same-as')
 
     def __init__(self, *args, **kwargs):
         super(HasSameAsForm, self).__init__(*args, **kwargs)
@@ -1144,7 +1156,7 @@ class HasSameAsForm(SearchFieldForm):
 class NoSameEmailForm(SearchFieldForm):
     """Allow same as contact in results"""
     name = 'no_same_email'
-    label = _(u'Duplicate emails')
+    label = _('Duplicate emails')
 
     def __init__(self, *args, **kwargs):
         super(NoSameEmailForm, self).__init__(*args, **kwargs)
@@ -1181,10 +1193,49 @@ class NoSameEmailForm(SearchFieldForm):
         return filtered_contacts
 
 
+class DuplicatedContactsForm(SearchFieldForm):
+    """Allow same as contact in results"""
+    name = 'duplicated_contacts'
+    label = _('Duplicated contacts')
+
+    def __init__(self, *args, **kwargs):
+        super(DuplicatedContactsForm, self).__init__(*args, **kwargs)
+        choices = ((1, _('same firstname and lastname')), (2, _('same lastname')), )
+        field = forms.ChoiceField(choices=choices, label=self.label)
+        self._add_field(field)
+
+    def get_lookup(self):
+        """lookup"""
+        pass
+
+    def get_exclude_lookup(self):
+        """exclude lookup"""
+        pass
+
+    def global_post_process(self, contacts):
+        """this filters the full list of results"""
+        keys = {}
+        filtered_contacts = []
+        try:
+            value = int(self.value)
+        except ValueError:
+            value = 1
+        for contact in contacts:
+            if contact.lastname:  # ignore contacts without names
+                key = contact.lastname if value == 2 else contact.fullname   # lastname only or fullname
+                if key:
+                    if key in keys:
+                        filtered_contacts.append(contact)
+                        filtered_contacts.append(keys[key])
+                    else:
+                        keys[key] = contact
+        return filtered_contacts
+
+
 class ContactsImportSearchForm(SearchFieldForm):
     """by import"""
     name = 'contact_import'
-    label = _(u'Import')
+    label = _('Import')
     
     def __init__(self, *args, **kwargs):
         super(ContactsImportSearchForm, self).__init__(*args, **kwargs)
@@ -1210,7 +1261,7 @@ class ByUserBaseFormSearchForm(SearchFieldForm):
 class ContactsAndEntitiesModifiedBySearchForm(ByUserBaseFormSearchForm):
     """by import"""
     name = 'contacts_and_entities_modified_by'
-    label = _(u'Contacts and entities modified by')
+    label = _('Contacts and entities modified by')
 
     def get_lookup(self):
         """lookup"""
@@ -1220,7 +1271,7 @@ class ContactsAndEntitiesModifiedBySearchForm(ByUserBaseFormSearchForm):
 class ContactsModifiedBySearchForm(ByUserBaseFormSearchForm):
     """by import"""
     name = 'contacts_modified_by'
-    label = _(u'Contacts and entities modified by')
+    label = _('Contacts and entities modified by')
 
     def get_lookup(self):
         """lookup"""
@@ -1230,7 +1281,7 @@ class ContactsModifiedBySearchForm(ByUserBaseFormSearchForm):
 class EntitiesModifiedBySearchForm(ByUserBaseFormSearchForm):
     """by import"""
     name = 'entities_modified_by'
-    label = _(u'Contacts and entities modified by')
+    label = _('Contacts and entities modified by')
 
     def get_lookup(self):
         """lookup"""
@@ -1240,7 +1291,7 @@ class EntitiesModifiedBySearchForm(ByUserBaseFormSearchForm):
 class EntityByModifiedDate(TwoDatesForm):
     """every entity modified betwwen two dates"""
     name = 'entity_by_modified_date'
-    label = _(u'Entity by modified date')
+    label = _('Entity by modified date')
 
     def get_lookup(self):
         """lookup"""
@@ -1252,7 +1303,7 @@ class EntityByModifiedDate(TwoDatesForm):
 class ContactByModifiedDate(TwoDatesForm):
     """every contact modified between two dates"""
     name = 'contact_by_modified_date'
-    label = _(u'Contact by modified date')
+    label = _('Contact by modified date')
 
     def get_lookup(self):
         """lookup"""
@@ -1264,7 +1315,7 @@ class ContactByModifiedDate(TwoDatesForm):
 class ContactsByCreationDate(TwoDatesForm):
     """by creation date"""
     name = 'contacts_by_creation_date'
-    label = _(u'Contacts by creation date')
+    label = _('Contacts by creation date')
     
     def get_lookup(self):
         """lookup"""
@@ -1275,7 +1326,7 @@ class ContactsByCreationDate(TwoDatesForm):
 class EntitiesByCreationDate(TwoDatesForm):
     """by entity creation date"""
     name = 'entities_by_creation_date'
-    label = _(u'Entities by creation date')
+    label = _('Entities by creation date')
     
     def get_lookup(self):
         """lookup"""
@@ -1288,7 +1339,7 @@ class EntitiesByCreationDate(TwoDatesForm):
 class ContactsAndEntitiesByChangeDate(TwoDatesForm):
     """by change date"""
     name = 'contacts_and_entities_by_change_date'
-    label = _(u'Contacts and entities by change date')
+    label = _('Contacts and entities by change date')
     
     def get_lookup(self):
         """lookup"""
@@ -1302,7 +1353,7 @@ class ContactsAndEntitiesByChangeDate(TwoDatesForm):
 class ContactsRelationshipByType(SearchFieldForm):
     """by type of relationship"""
     name = 'contacts_by_relationship_type'
-    label = _(u'Relationship type')
+    label = _('Relationship type')
     
     def __init__(self, *args, **kwargs):
         super(ContactsRelationshipByType, self).__init__(*args, **kwargs)
@@ -1336,7 +1387,7 @@ class ContactsRelationshipByType(SearchFieldForm):
 class ContactsRelationshipByDate(TwoDatesForm):
     """by relationship date"""
     name = 'contacts_by_relationship_dates'
-    label = _(u'Relationship dates')
+    label = _('Relationship dates')
         
     def get_lookup(self):
         """lookup"""
@@ -1353,7 +1404,7 @@ class ContactsRelationshipByDate(TwoDatesForm):
 class ContactWithCustomField(SearchFieldForm):
     """by contact with custom field"""
     name = 'contact_with_custom_field'
-    label = _(u'Contacts with custom field')
+    label = _('Contacts with custom field')
     
     def __init__(self, *args, **kwargs):
         super(ContactWithCustomField, self).__init__(*args, **kwargs)
@@ -1373,7 +1424,7 @@ class ContactWithCustomField(SearchFieldForm):
 class EntityWithCustomField(SearchFieldForm):
     """by entity custom field"""
     name = 'entity_with_custom_field'
-    label = _(u'Entities with custom field')
+    label = _('Entities with custom field')
     
     def __init__(self, *args, **kwargs):
         super(EntityWithCustomField, self).__init__(*args, **kwargs)
@@ -1445,16 +1496,17 @@ class UnitTestContactCustomFieldForm(CustomFieldBaseSearchForm):
 class SortContacts(SearchFieldForm):
     """sort contacts"""
     name = 'sort'
-    label = _(u'Sort contacts')
+    label = _('Sort contacts')
     contacts_display = True
+    is_sort_form = True
     
     def __init__(self, *args, **kwargs):
         super(SortContacts, self).__init__(*args, **kwargs)
         choices = (
-            ('name', _(u'Name')),
-            ('entity', _(u'Entity')),
-            ('contact', _(u'Contact')),
-            ('zipcode', _(u'Zipcode')),
+            ('entity', _('Entity name')),
+            ('contact', _('Contact name')),
+            ('name', _('Entity or Contact name')),
+            ('zipcode', _('Zipcode')),
         ) 
         field = forms.CharField(
             label=self.label,
@@ -1468,36 +1520,26 @@ class SortContacts(SearchFieldForm):
         )
         self._add_field(field)
         self.default_country = get_default_country()
-    
+
     def _sort_by_name(self, contact):
-        """sort by name"""
-        if contact.entity.is_single_contact:
-            value = u"{0} {1}".format(contact.lastname, contact.firstname)
-        else:
-            value = contact.entity.name
-        return value.upper()
-    
-    def _sort_by_contact(self, contact):
-        """sort by contact name"""
-        return u"{0} {1}".format(contact.lastname, contact.firstname).upper()
-    
+        return sort_by_name_callback(contact)
+
     def _sort_by_entity(self, contact):
-        """sort by entity"""
-        value1 = u"B" if contact.entity.is_single_contact else u"A"
-        value2 = self._sort_by_name(contact)
-        value3 = u"{0} {1}".format(contact.lastname, contact.firstname) if not contact.entity.is_single_contact else u""
-        return value1, value2, value3
-    
+        return sort_by_entity_callback(contact)
+
+    def _sort_by_contact(self, contact):
+        return sort_by_contact_callback(contact)
+
     def _sort_by_zipcode(self, contact):
         """sort by zipcode"""
         country = contact.get_country()
-        value1 = contact.get_zip_code or u'?'
+        value1 = contact.get_zip_code or '?'
         city = contact.get_city
         if not city:
             prefix1 = "C"
             prefix2 = value2 = "?"
         else:
-            prefix1 = u"A" if ((not country) or self.default_country.id == country.id) else u"B"
+            prefix1 = "A" if ((not country) or self.default_country.id == country.id) else "B"
             prefix2 = country.name if country else self.default_country.name
             value2 = city.name
         value3 = self._sort_by_name(contact)
@@ -1511,3 +1553,45 @@ class SortContacts(SearchFieldForm):
         """filter the final results"""
         callback = getattr(self, '_sort_by_{0}'.format(self.value), None)
         return sorted(contacts, key=callback)
+
+
+class ContactWithEmailInGroupSearchForm(SearchFieldForm):
+    """get contacts with someone with the same email in group"""
+    name = 'email_in_group'
+    label = _(u'Some with same email in group')
+
+    def _get_widget(self):
+        """customize widget: autocomplete"""
+        return GroupAutoComplete(attrs={
+            'placeholder': _(u'Enter part of the group name'), 'size': '80',
+        })
+
+    def __init__(self, *args, **kwargs):
+        super(ContactWithEmailInGroupSearchForm, self).__init__(*args, **kwargs)
+        queryset = models.Group.objects.all()
+        try:
+            queryset = queryset.extra(select={'lower_name': 'lower(name)'}).order_by('lower_name')
+        except Exception:  # pylint: disable=broad-except
+            queryset = queryset.order_by('name')
+        kwargs = {}
+        widget = self._get_widget()
+        if widget:
+            kwargs['widget'] = widget
+        field = forms.ModelChoiceField(queryset, label=self.label, **kwargs)
+        self._add_field(field)
+
+    # def get_lookup(self):
+    #     """lookup"""
+    #     return Q(entity__group__id=self.value) | Q(group__id=self.value)
+
+    def global_post_process(self, contacts):
+        """filter the final results"""
+        group_contacts = models.Contact.objects.filter(Q(entity__group__id=self.value) | Q(group__id=self.value))
+
+        contacts_with_same_email_in_group = []
+        for contact in contacts:
+            email = contact.get_email
+            if group_contacts.filter(Q(email=email) | Q(email='', entity__email=email)):
+                contacts_with_same_email_in_group.append(contact)
+
+        return contacts_with_same_email_in_group
