@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 """unit testing"""
 
-from django.core.urlresolvers import reverse
+from django.urls import reverse
 
 from coop_cms.tests import BeautifulSoup
 from model_mommy import mommy
