@@ -2,11 +2,12 @@
 """django-modeltranslation configuration"""
 
 from modeltranslation.translator import translator, TranslationOptions
-from models import EntityType
+
+from .models import EntityType
 
 
 class EntityTypeTranslationOptions(TranslationOptions):
     fields = ('name',)
 
-translator.register(EntityType, EntityTypeTranslationOptions)
 
+translator.register(EntityType, EntityTypeTranslationOptions)
