@@ -3,12 +3,13 @@
 
 import json
 
-from django.core.urlresolvers import reverse
 from django.contrib.auth.decorators import user_passes_test
-from django.db.models import Q
+from django.contrib.sites.models import Site
+from django.db.models import Q, ObjectDoesNotExist
 from django.http import HttpResponse, Http404, HttpResponseRedirect
-from django.shortcuts import render_to_response, render, get_object_or_404
-from django.template import RequestContext, Template, Context
+from django.shortcuts import render, get_object_or_404
+from django.template import Template, Context
+from django.urls import reverse
 from django.utils.translation import ugettext as _
 
 from colorbox.decorators import popup_redirect, popup_reload, popup_close
@@ -76,14 +77,14 @@ def view_all_contact_actions(request, contact_id, action_set_id):
     actions = contact.action_set.filter(archived=False).order_by("planned_date", "priority")
     actions_by_set = get_actions_by_set(actions, 0, action_set_list)
 
-    return render_to_response(
+    return render(
+        request,
         'Crm/view_contact_actions.html',
         {
             'contact': contact,
             'actions_by_set': actions_by_set,
             'entity': contact.entity
-        },
-        context_instance=RequestContext(request)
+        }
     )
 
 
@@ -104,13 +105,13 @@ def view_all_entity_actions(request, entity_id, action_set_id):
     ).distinct().order_by("planned_date", "priority")
     actions_by_set = get_actions_by_set(actions, 0, action_set_list)
 
-    return render_to_response(
+    return render(
+        request,
         'Crm/view_entity_actions.html',
         {
             'actions_by_set': actions_by_set,
             'entity': entity
-        },
-        context_instance=RequestContext(request)
+        }
     )
 
 
@@ -153,10 +154,10 @@ def add_action_for_contact(request, contact_id):
         'contact': contact,
     }
 
-    return render_to_response(
+    return render(
+        request,
         'Crm/edit_action.html',
-        context,
-        context_instance=RequestContext(request)
+        context
     )
 
 
@@ -172,7 +173,7 @@ def view_entity_actions(request, entity_id, set_id):
         title = action_set.name
     else:
         filters.append(Q(type__set=None))
-        title = _(u"Other kind of actions") if models.ActionSet.objects.count() else _(u"Actions")
+        title = _("Other kind of actions") if models.ActionSet.objects.count() else _("Actions")
 
     actions = models.Action.objects.filter(
         Q(entity=entity) | Q(contact__entity=entity) | Q(opportunity__entity=entity), *filters
@@ -180,7 +181,8 @@ def view_entity_actions(request, entity_id, set_id):
     request.session["redirect_url"] = reverse('crm_entity_actions', args=[entity_id, set_id])
     page_obj = paginate(request, actions, 50)
 
-    return render_to_response(
+    return render(
+        request,
         'Crm/entity_actions.html',
         {
             'title': title,
@@ -190,8 +192,7 @@ def view_entity_actions(request, entity_id, set_id):
             'actions': list(page_obj),
             'page_obj': page_obj,
             'filters': filters,
-        },
-        context_instance=RequestContext(request)
+        }
     )
 
 
@@ -207,13 +208,14 @@ def view_contact_actions(request, contact_id, set_id):
         title = action_set.name
     else:
         filters.append(Q(type__set=None))
-        title = _(u"Other kind of actions") if models.ActionSet.objects.count() else _(u"Actions")
+        title = _("Other kind of actions") if models.ActionSet.objects.count() else _("Actions")
 
     actions = contact.action_set.filter(*filters).order_by("planned_date", "priority")
     page_obj = paginate(request, actions, 50)
     request.session["redirect_url"] = reverse('crm_contact_actions', args=[contact_id, set_id])
 
-    return render_to_response(
+    return render(
+        request,
         'Crm/entity_actions.html',
         {
             'contact': contact,
@@ -222,14 +224,13 @@ def view_contact_actions(request, contact_id, set_id):
             'page_obj': page_obj,
             'all_actions': True,
             'title': title,
-        },
-        context_instance=RequestContext(request)
+        }
     )
 
 
 @user_passes_test(can_access)
 @popup_redirect
-def create_action(request, entity_id, contact_id):
+def create_action(request, entity_id, contact_id, type_id=None):
     """view"""
     # entity_id or contact_id can be 0
     # add from menu -> both are 0 / add from contact -> entity_id = 0 / add from entity -> contact_id = 0
@@ -238,8 +239,13 @@ def create_action(request, entity_id, contact_id):
     contact = get_object_or_404(models.Contact, id=contact_id) if contact_id else None
     entity = get_object_or_404(models.Entity, id=entity_id) if entity_id else None
 
+    try:
+        action_type = models.ActionType.objects.get(id=type_id)
+    except (ValueError, models.ActionType.DoesNotExist):
+        action_type = None
+
     if request.method == 'POST':
-        form = forms.ActionForm(request.POST)
+        form = forms.ActionForm(request.POST, action_type=action_type)
         if form.is_valid():
             next_url = request.session.get('redirect_url')
             action = form.save()
@@ -272,24 +278,19 @@ def create_action(request, entity_id, contact_id):
         except (ValueError, models.Opportunity.DoesNotExist):
             pass
 
-        try:
-            type_id = int(request.GET.get('type', 0))
-            initial['type'] = models.ActionType.objects.get(id=type_id)
-        except (ValueError, models.ActionType.DoesNotExist):
-            pass
-
-        form = forms.ActionForm(initial=initial)
+        form = forms.ActionForm(initial=initial, action_type=action_type)
 
     context = {
         'form': form,
         'contact_id': contact_id,
         'entity_id': entity_id,
+        'type_id': action_type.id if action_type else 0,
     }
 
-    return render_to_response(
+    return render(
+        request,
         'Crm/edit_action.html',
-        context,
-        context_instance=RequestContext(request)
+        context
     )
 
 
@@ -314,10 +315,10 @@ def edit_action(request, action_id):
         'action': action,
     }
 
-    return render_to_response(
+    return render(
+        request,
         'Crm/edit_action.html',
-        context,
-        context_instance=RequestContext(request)
+        context
     )
 
 
@@ -345,14 +346,14 @@ def delete_action(request, action_id):
     else:
         form = forms.ConfirmForm()
 
-    return render_to_response(
+    return render(
+        request,
         'balafon/confirmation_dialog.html',
         {
             'form': form,
-            'message': _(u'Are you sure to delete this action?'),
+            'message': _('Are you sure to delete this action?'),
             'action_url': reverse("crm_delete_action", args=[action_id]),
-        },
-        context_instance=RequestContext(request)
+        }
     )
 
 
@@ -361,7 +362,8 @@ def view_all_actions(request):
     """view"""
     actions = models.Action.objects.all().order_by("-planned_date")
     request.session["redirect_url"] = reverse('crm_all_actions')
-    return render_to_response(
+    return render(
+        request,
         'Crm/all_actions.html',
         {
             'actions': actions,
@@ -370,8 +372,7 @@ def view_all_actions(request):
             'default_my_actions': False,
             'all_actions': True,
             'view_name': "crm_all_actions",
-        },
-        context_instance=RequestContext(request)
+        }
     )
 
 
@@ -389,10 +390,10 @@ def do_action(request, action_id):
     else:
         form = forms.ActionDoneForm(instance=action)
 
-    return render_to_response(
+    return render(
+        request,
         'Crm/do_action.html',
-        {'form': form, 'action': action},
-        context_instance=RequestContext(request)
+        {'form': form, 'action': action}
     )
 
 
@@ -417,11 +418,11 @@ def reassign_action(request, action_id):
     else:
         form = forms.SelectContactOrEntityForm()
 
-    title = _(u'Reassign {0}'.format(action.type.name.lower()) if action.type else _(u'action'))
-    return render_to_response(
+    title = _('Reassign {0}'.format(action.type.name.lower()) if action.type else _('action'))
+    return render(
+        request,
         'Crm/popup_reassign_action.html',
-        {'form': form, 'action': action, 'title': title},
-        context_instance=RequestContext(request)
+        {'form': form, 'action': action, 'title': title}
     )
 
 
@@ -441,10 +442,10 @@ def add_contact_to_action(request, action_id):
     else:
         form = forms.SelectContactForm()
 
-    return render_to_response(
+    return render(
+        request,
         "Crm/add_contact_to_action.html",
-        {'form': form, 'action': action},
-        context_instance=RequestContext(request)
+        {'form': form, 'action': action}
     )
 
 
@@ -465,14 +466,14 @@ def remove_contact_from_action(request, action_id, contact_id):
     else:
         form = forms.ConfirmForm()
 
-    return render_to_response(
+    return render(
+        request,
         'balafon/confirmation_dialog.html',
         {
             'form': form,
-            'message': _(u'Do you to remove {0} from this action?').format(contact),
+            'message': _('Do you to remove {0} from this action?').format(contact),
             'action_url': reverse("crm_remove_contact_from_action", args=[action_id, contact_id]),
-        },
-        context_instance=RequestContext(request)
+        }
     )
 
 
@@ -492,10 +493,10 @@ def add_entity_to_action(request, action_id):
     else:
         form = forms.SelectEntityForm()
 
-    return render_to_response(
+    return render(
+        request,
         "Crm/add_entity_to_action.html",
-        {'form': form, 'action': action},
-        context_instance=RequestContext(request)
+        {'form': form, 'action': action}
     )
 
 
@@ -516,14 +517,14 @@ def remove_entity_from_action(request, action_id, entity_id):
     else:
         form = forms.ConfirmForm()
 
-    return render_to_response(
+    return render(
+        request,
         'balafon/confirmation_dialog.html',
         {
             'form': form,
-            'message': _(u'Do you to remove {0} from this action?').format(entity),
+            'message': _('Do you to remove {0} from this action?').format(entity),
             'action_url': reverse("crm_remove_entity_from_action", args=[action_id, entity_id]),
-        },
-        context_instance=RequestContext(request)
+        }
     )
 
 
@@ -581,17 +582,15 @@ def update_action_status(request, action_id, status2=False):
 
         form = form_class(request.POST, instance=action)
         if form.is_valid():
-            if status2:
-                action.status2 = form.cleaned_data['status2']
-            else:
+            if not status2:
                 action.status = form.cleaned_data['status']
+            action.status2 = form.cleaned_data['status2']
             action.save()
-
             return None
     else:
 
         # When the popup is shown after clicking on "done" button
-        # show a final status has default value
+        # show a final status as default value
         on_do = request.GET.get('done', None)
         if on_do and action.type:
             if not status2:
@@ -622,23 +621,51 @@ def mailto_action(request, action_id):
     """Open the mail client in order to send email to action contacts"""
 
     action = get_object_or_404(models.Action, id=action_id)
-    mailto_settings = get_object_or_404(models.MailtoSettings, action_type=action.type)
+    try:
+        mailto_settings = models.MailtoSettings.objects.get(action_type=action.type)
+        body_template = mailto_settings.body_template
+        subject = mailto_settings.subject or action.type.mail_to_subject or action.subject
+        is_bcc = mailto_settings.bcc
+    except models.MailtoSettings.DoesNotExist:
+        body_template = ''
+        mailto_settings = None
+        if action.type:
+            subject = action.type.mail_to_subject or action.subject
+        else:
+            subject = action.subject
+        is_bcc = False
+
+    if action.status and action.status.next_on_send:
+        action.status = action.status.next_on_send
+        action.save()
 
     emails = [contact.get_email for contact in action.contacts.all()]
     emails += [entity.email for entity in action.entities.all()]
 
-    template_text = Template(mailto_settings.body_template)
-    body = template_text.render(Context({'action': action, 'settings': mailto_settings}))
+    doc_url = ''
+    if action.uuid and hasattr(action, 'sale'):
+        try:
+            url = reverse('store_view_sales_document_public', args=[action.uuid])
+            prefix = "http{0}://".format('s' if request.is_secure() else '')
+            doc_url = prefix + Site.objects.get_current().domain + url
+        except ObjectDoesNotExist:
+            pass
 
-    # Remove duplicates and empty and soty in alphabetical order
+    if body_template:
+        template_text = Template(body_template)
+        body = template_text.render(Context({'action': action, 'settings': mailto_settings, 'doc_url': doc_url}))
+    else:
+        body = ''
+
+    # Remove duplicates and empty and sort in alphabetical order
     emails = list(sorted(set([email for email in emails if email])))
 
-    mailto = u'mailto:'
-    if mailto_settings.bcc:
+    mailto = 'mailto:'
+    if is_bcc:
         mailto += '?bcc='
     mailto += ','.join(emails)
-    mailto += "&" if mailto_settings.bcc else "?"
-    mailto += 'subject={0}'.format(mailto_settings.subject or action.subject)
+    mailto += "&" if is_bcc else "?"
+    mailto += 'subject={0}'.format(subject)
     mailto += '&body={0}'.format(body)
     return HttpResponseRedirectMailtoAllowed(mailto)
 
