@@ -5,14 +5,13 @@ from datetime import datetime
 import os.path
 import re
 
-from django.core.urlresolvers import reverse
 from django.conf import settings
 from django.contrib.auth.decorators import user_passes_test
 from django.contrib.messages import success, error
 from django.db.models import Max
 from django.http import HttpResponse, HttpResponseRedirect
-from django.shortcuts import render_to_response, get_object_or_404
-from django.template import RequestContext
+from django.shortcuts import render, get_object_or_404
+from django.urls import reverse
 from django.utils.text import slugify
 from django.utils.translation import ugettext as _
 
@@ -38,10 +37,10 @@ def new_contacts_import(request):
     else:
         form = forms.ContactsImportForm()
 
-    return render_to_response(
+    return render(
+        request,
         'Crm/new_contacts_import.html',
-        {'form': form},
-        context_instance=RequestContext(request)
+        {'form': form}
     )
 
 
@@ -81,13 +80,13 @@ def _get_subcription_field_name(subscription_type):
 
 def _set_contact_and_entity(contact_data, entity_dict, extract_from_email):
     """format properly name and entity"""
-    name = u"< {0} >".format(_(u"Unknown"))
+    name = "< {0} >".format(_("Unknown"))
     email_providers = (
         'free', 'gmail', 'yahoo', 'yahoo.co', 'wanadoo', 'orange', 'sfr', 'laposte',
         'hotmail', 'neuf', 'club-internet', 'voila', 'aol', 'live', 'ymail', 'outlook',
     )
     if not contact_data['entity']:
-        entity = u''
+        entity = ''
         if extract_from_email:
             res = re.match(r'(?P<name>.+)@(?P<cpn>.+)\.(?P<ext>.+)', contact_data['email'])
             if res:
@@ -212,8 +211,7 @@ def read_contacts(reader, fields, extract_from_email):
 def get_imports_fields():
     """get the list of fields to import"""
     fields = [
-        'gender', 'firstname', 'lastname', 'email', 'phone', 'mobile', 'job',
-        'notes', 'role',
+        'gender', 'firstname', 'lastname', 'email', 'phone', 'mobile', 'job', 'notes', 'role',
     ]
 
     for subscription_type in models.SubscriptionType.objects.all():
@@ -221,7 +219,8 @@ def get_imports_fields():
             _get_subcription_field_name(subscription_type)
         ]
 
-    fields += [    'entity', 'entity.type', 'entity.description', 'entity.website', 'entity.email',
+    fields += [
+        'entity', 'entity.type', 'entity.description', 'entity.website', 'entity.email',
         'entity.phone', 'entity.fax', 'entity.notes',
         'entity.address', 'entity.address2', 'entity.address3',
         'entity.city', 'entity.cedex', 'entity.zip_code', 'entity.country',
@@ -229,22 +228,22 @@ def get_imports_fields():
         'entity.groups', 'groups', 'favorite_language', 'title', 'birth_date'
     ]
 
-    #custom fields
+    # custom fields
     custom_fields_count = models.CustomField.objects.all().aggregate(Max('import_order'))['import_order__max']
     if not custom_fields_count:
         custom_fields_count = 0
-    for i in xrange(custom_fields_count):
+    for i in range(custom_fields_count):
         fields.append('cf_{0}'.format(i+1))
 
     custom_fields = []
-    for index_ in xrange(1, custom_fields_count+1):
+    for index_ in range(1, custom_fields_count+1):
         try:
             custom_field = models.CustomField.objects.get(import_order=index_)
             custom_fields.append(custom_field)
         except models.CustomField.DoesNotExist:
             custom_fields.append(None)
         except models.CustomField.MultipleObjectsReturned:
-            raise Exception(_(u"There are several custom fields with index {0}").format(index_))
+            raise Exception(_("There are several custom fields with index {0}").format(index_))
 
     return fields, custom_fields
 
@@ -257,19 +256,19 @@ def contacts_import_template(request):
 
     cols = fields[:len(fields) - len(custom_fields)] + custom_fields
 
-    template_file = u";".join([u'"{0}"'.format(unicode(col)) for col in cols])+u"\n"
+    template_file = ";".join(['"{0}"'.format('{0}'.format(col)) for col in cols]) + "\n"
 
     return HttpResponse(template_file, content_type="text/csv", )
 
 
 def _create_contact(contact_data, contacts_import, entity_dict):
     """create a contact from import file"""
-    #Entity
+    # Entity
     if settings.DEBUG:
         try:
-            print contact_data['entity'], contact_data['lastname']
+            print(contact_data['entity'], contact_data['lastname'])
         except UnicodeError:
-            print '##!'
+            print('##!')
 
     if not contact_data['entity.type']:
         entity_type = contacts_import.entity_type
@@ -288,7 +287,7 @@ def _create_contact(contact_data, contacts_import, entity_dict):
                 contact__firstname=contact_data['firstname'],
                 is_single_contact=True
             )[0]
-            entity.name = u"{0} {1}".format(contact_data['firstname'], contact_data['lastname'])
+            entity.name = "{0} {1}".format(contact_data['firstname'], contact_data['lastname'])
             entity.imported_by = contacts_import
             entity.save()
 
@@ -298,7 +297,7 @@ def _create_contact(contact_data, contacts_import, entity_dict):
         is_first_for_entity = entity.name not in entity_dict
         entity_dict[entity.name] = True
 
-    #Contact
+    # Contact
     contact = models.Contact.objects.get_or_create(
         entity=entity, firstname=contact_data['firstname'], lastname=contact_data['lastname']
     )[0]
@@ -428,7 +427,7 @@ def confirm_contacts_import(request, import_id):
     fields, custom_fields = get_imports_fields()
 
     custom_fields_count = len(custom_fields)
-    cf_names = ['cf_{0}'.format(idx) for idx in xrange(1, custom_fields_count+1)]
+    cf_names = ['cf_{0}'.format(idx) for idx in range(1, custom_fields_count+1)]
 
     contacts_import = get_object_or_404(models.ContactsImport, id=import_id)
     try:
@@ -473,22 +472,22 @@ def confirm_contacts_import(request, import_id):
             )
             contacts, total_contacts = read_contacts(reader, fields, contacts_import.entity_name_from_email)
 
-        return render_to_response(
+        return render(
+            request,
             'Crm/confirm_contacts_import.html',
-            {'form': form, 'contacts': contacts, 'nb_contacts': len(contacts), 'total_contacts': total_contacts},
-            context_instance=RequestContext(request)
+            {'form': form, 'contacts': contacts, 'nb_contacts': len(contacts), 'total_contacts': total_contacts}
         )
 
     except UnicodeDecodeError:
         error(
             request,
-            _(u"An error occurred while reading the csv file. You should check that the file encoding is correct.")
+            _("An error occurred while reading the csv file. You should check that the file encoding is correct.")
         )
-    except Exception, msg:  # pylint: disable broad-except
+    except Exception as msg:  # pylint: disable broad-except
         error(
             request,
-            u'{0}: {1}'.format(
-                _(u"An error occurred. You should check that the file is a valid csv file."),
+            '{0}: {1}'.format(
+                _("An error occurred. You should check that the file is a valid csv file."),
                 msg
             )
         )
@@ -519,17 +518,17 @@ def unsubscribe_contacts_import(request):
                             subscription.accept_subscription = False
                             subscription.unsubscription_date = datetime.now()
                             subscription.save()
-                success(request, _(u"{0} contacts have been unsubscribed").format(contacts_count))
+                success(request, _("{0} contacts have been unsubscribed").format(contacts_count))
 
             except UnicodeDecodeError:
-                error(request, _(u"Unicode error while reading the file"))
+                error(request, _("Unicode error while reading the file"))
 
             return HttpResponseRedirect(reverse('balafon_homepage'))
     else:
         form = forms.UnsubscribeContactsImportForm()
 
-    return render_to_response(
+    return render(
+        request,
         'Crm/unsubscribe_contacts_import.html',
         {'form': form},
-        context_instance=RequestContext(request)
     )
