@@ -8,8 +8,8 @@ from django.conf import settings
 from django.contrib.sites.models import Site
 from django.core import management
 from django.core import mail
-from django.core.urlresolvers import reverse
 from django.test.utils import override_settings
+from django.urls import reverse
 from django.utils.translation import activate
 
 from coop_cms.models import Newsletter
@@ -79,7 +79,7 @@ class SendEmailingTest(BaseTestCase):
             emailing.send_to.add(contact)
         emailing.save()
 
-        management.call_command('emailing_scheduler', verbosity=0, interactive=False)
+        management.call_command('emailing_scheduler', verbosity=0)
 
         emailing = Emailing.objects.get(id=emailing.id)
 
@@ -133,6 +133,104 @@ class SendEmailingTest(BaseTestCase):
             self.assertEqual(c.action_set.count(), 1)
             self.assertEqual(c.action_set.all()[0].subject, email.subject)
 
+    @override_settings(COOP_CMS_REPLY_TO="")
+    def test_send_newsletter_avoid_magic(self):
+
+        entity = mommy.make(models.Entity, name="my corp")
+
+        names = ['alpha', 'beta', 'gamma']
+        contacts = [
+            mommy.make(
+                models.Contact, entity=entity,
+                email=name + '@toto.fr', lastname=name.capitalize()
+            ) for name in names
+        ]
+
+        content = '<h2>Hello #!-fullname-!#!</h2><p>Visit <a href="http://toto.fr">us</a>'
+        content += '<a href="mailto:me@me.fr">mailme</a>'
+        content += '<a href="tel:123">callme</a>'
+        content += '<a href="#art1">internal link</a></p>'
+
+        newsletter_data = {
+            'subject': 'This is the subject',
+            'content': content,
+            'template': 'test/newsletter_contact.html'
+        }
+        newsletter = mommy.make(Newsletter, **newsletter_data)
+
+        site = Site.objects.get_current()
+        site.domain = "toto.fr"
+        site.save()
+
+        emailing = mommy.make(
+            Emailing,
+            newsletter=newsletter,
+            status=Emailing.STATUS_SCHEDULED,
+            scheduling_dt=datetime.now(),
+            sending_dt=None,
+            subscription_type=mommy.make(models.SubscriptionType, site=site)
+        )
+
+        for contact in contacts:
+            emailing.send_to.add(contact)
+        emailing.save()
+
+        management.call_command('emailing_scheduler', verbosity=0)
+
+        emailing = Emailing.objects.get(id=emailing.id)
+
+        # check emailing status
+        self.assertEqual(emailing.status, Emailing.STATUS_SENT)
+        self.assertNotEqual(emailing.sending_dt, None)
+        self.assertEqual(emailing.send_to.count(), 0)
+        self.assertEqual(emailing.sent_to.count(), len(contacts))
+
+        self.assertEqual(len(mail.outbox), len(contacts))
+
+        outbox = list(mail.outbox)
+        outbox.sort(key=lambda e: e.to)
+        contacts.sort(key=lambda c: c.get_email)
+
+        for email, contact in zip(outbox, contacts):
+            viewonline_url = emailing.get_domain_url_prefix() + reverse(
+                'emailing_view_online', args=[emailing.id, contact.uuid]
+            )
+            unsubscribe_url = emailing.get_domain_url_prefix() + reverse(
+                'emailing_unregister', args=[emailing.id, contact.uuid]
+            )
+
+            self.assertEqual(email.to, [contact.get_email_address()])
+            self.assertEqual(email.from_email, settings.COOP_CMS_FROM_EMAIL)
+            self.assertEqual(email.subject, newsletter_data['subject'])
+            self.assertTrue(email.body.find(entity.name) >= 0)
+            self.assertEqual(email.extra_headers.get('Reply-To', ''), '')
+            self.assertEqual(
+                email.extra_headers['List-Unsubscribe'],
+                '<{0}>, <mailto:{1}?subject=unsubscribe>'.format(unsubscribe_url, email.from_email)
+            )
+
+            self.assertTrue(email.body.find(contact.fullname) >= 0)
+            self.assertTrue(email.alternatives[0][1], "text/html")
+
+            self.assertTrue(email.alternatives[0][0].find(contact.fullname) >= 0)
+            self.assertTrue(email.alternatives[0][0].find(entity.name) >= 0)
+            self.assertTrue(email.alternatives[0][0].find(viewonline_url) >= 0)
+            self.assertTrue(email.alternatives[0][0].find(unsubscribe_url) >= 0)
+            # Check mailto links are not magic
+            self.assertTrue(email.alternatives[0][0].find("mailto:me@me.fr") > 0)
+            # Check tel links are not magic
+            self.assertTrue(email.alternatives[0][0].find("tel:123") > 0)
+            # Check anchor links are not magic
+            self.assertTrue(email.alternatives[0][0].find("#art1") > 0)
+
+            # check magic links
+            self.assertTrue(MagicLink.objects.count() > 0)
+
+            # check an action has been created
+            c = models.Contact.objects.get(id=contact.id)
+            self.assertEqual(c.action_set.count(), 1)
+            self.assertEqual(c.action_set.all()[0].subject, email.subject)
+
     @skipIf(len(settings.LANGUAGES) < 2, "LANGUAGES less than 2")
     @override_settings(COOP_CMS_REPLY_TO="")
     def test_send_newsletter_language(self):
@@ -183,7 +281,7 @@ class SendEmailingTest(BaseTestCase):
             emailing.send_to.add(contact)
         emailing.save()
 
-        management.call_command('emailing_scheduler', verbosity=0, interactive=False)
+        management.call_command('emailing_scheduler', verbosity=0)
 
         emailing = Emailing.objects.get(id=emailing.id)
 
@@ -297,7 +395,7 @@ class SendEmailingTest(BaseTestCase):
             emailing.send_to.add(contact)
         emailing.save()
 
-        management.call_command('emailing_scheduler', verbosity=0, interactive=False)
+        management.call_command('emailing_scheduler', verbosity=0)
 
         emailing = Emailing.objects.get(id=emailing.id)
 
@@ -404,7 +502,7 @@ class SendEmailingTest(BaseTestCase):
             emailing.send_to.add(contact)
         emailing.save()
 
-        management.call_command('emailing_scheduler', verbosity=0, interactive=False)
+        management.call_command('emailing_scheduler', verbosity=0)
 
         emailing = Emailing.objects.get(id=emailing.id)
 
@@ -504,7 +602,7 @@ class SendEmailingTest(BaseTestCase):
             emailing.send_to.add(contact)
         emailing.save()
 
-        management.call_command('emailing_scheduler', verbosity=0, interactive=False)
+        management.call_command('emailing_scheduler', verbosity=0)
 
         emailing = Emailing.objects.get(id=emailing.id)
 
@@ -602,7 +700,7 @@ class SendEmailingTest(BaseTestCase):
             emailing.send_to.add(contact)
         emailing.save()
 
-        management.call_command('emailing_scheduler', verbosity=0, interactive=False)
+        management.call_command('emailing_scheduler', verbosity=0)
 
         emailing = Emailing.objects.get(id=emailing.id)
 
@@ -879,7 +977,7 @@ class SendEmailingTest(BaseTestCase):
         redirect_url = other_lang + next_url
         self.assertTrue(response['Location'].find(redirect_url) >= 0)
 
-    @override_settings(SECRET_KEY=u"super-héros")
+    @override_settings(SECRET_KEY="super-héros")
     def test_view_online_utf_links(self):
         entity = mommy.make(models.Entity, name="my corp")
         contact = mommy.make(
@@ -888,7 +986,7 @@ class SendEmailingTest(BaseTestCase):
         )
         newsletter_data = {
             'subject': 'This is the subject',
-            'content': u'<h2>Hello #!-fullname-!#!</h2><p>Visit <a href="http://toto.fr/à-bientôt">à bientôt</a></p>',
+            'content': '<h2>Hello #!-fullname-!#!</h2><p>Visit <a href="http://toto.fr/à-bientôt">à bientôt</a></p>',
             'template': 'test/newsletter_contact.html'
         }
         newsletter = mommy.make(Newsletter, **newsletter_data)
@@ -909,7 +1007,7 @@ class SendEmailingTest(BaseTestCase):
 
         magic_link1 = MagicLink.objects.all()[1]
         self.assertContains(response, reverse('emailing_view_link', args=[magic_link1.uuid, contact.uuid]))
-        self.assertEqual(magic_link1.url, u'http://toto.fr/à-bientôt')
+        self.assertEqual(magic_link1.url, 'http://toto.fr/à-bientôt')
 
     def test_view_long_links(self):
         entity = mommy.make(models.Entity, name="my corp")
@@ -921,7 +1019,7 @@ class SendEmailingTest(BaseTestCase):
         long_link = "http://toto.fr/{0}".format("abcde" * 100)  # >500 chars
         newsletter_data = {
             'subject': 'This is the subject',
-            'content': u'<p>Visit <a href="{0}">long link</a> <a href="{1}">long link</a></p>'.format(
+            'content': '<p>Visit <a href="{0}">long link</a> <a href="{1}">long link</a></p>'.format(
                 short_link, long_link),
             'template': 'test/newsletter_no_link.html'
         }
@@ -951,7 +1049,7 @@ class SendEmailingTest(BaseTestCase):
         short_link = "http://toto.fr/abcde/"
         newsletter_data = {
             'subject': 'This is the subject',
-            'content': u'<p>Visit <a href="{0}">link1</a> <a href="{1}">link2</a></p>'.format(
+            'content': '<p>Visit <a href="{0}">link1</a> <a href="{1}">link2</a></p>'.format(
                 short_link, short_link
             ),
             'template': 'test/newsletter_no_link.html'
@@ -980,7 +1078,7 @@ class SendEmailingTest(BaseTestCase):
         short_link = "http://toto.fr/abcde/"
         newsletter_data = {
             'subject': 'This is the subject',
-            'content': u'<p>Visit <a href="{0}">link1</a></p>'.format(
+            'content': '<p>Visit <a href="{0}">link1</a></p>'.format(
                 short_link
             ),
             'template': 'test/newsletter_no_link.html'
@@ -1030,8 +1128,8 @@ class CustomTemplateTest(BaseTestCase):
             gender=models.Contact.GENDER_MALE,
             entity=entity,
             email='john@toto.fr',
-            lastname=u'Doe',
-            firstname=u'John',
+            lastname='Doe',
+            firstname='John',
             address='2 rue de la lune',
             zip_code='42424',
             city=city
@@ -1069,7 +1167,7 @@ class CustomTemplateTest(BaseTestCase):
         emailing.send_to.add(contact)
         emailing.save()
 
-        management.call_command('emailing_scheduler', verbosity=0, interactive=False)
+        management.call_command('emailing_scheduler', verbosity=0)
 
         emailing = Emailing.objects.get(id=emailing.id)
 
@@ -1109,25 +1207,25 @@ class MaxLineLengthTest(BaseTestCase):
         """it should not have a line more than 400 chars"""
 
         in_lines = [
-            u'This is a messsage',
-            u"123456789 " * 50,
-            u"That's all folks!"
+            'This is a messsage',
+            "123456789 " * 50,
+            "That's all folks!"
         ]
-        text = u"\n".join(in_lines)
+        text = "\n".join(in_lines)
 
         out_text = force_line_max_length(text, max_length_per_line=400, dont_cut_in_quotes=True)
-        out_lines = out_text.split(u"\n")
+        out_lines = out_text.split("\n")
 
         self.assertEqual(len(out_lines), 4)
         self.assertEqual(out_lines[0], in_lines[0])
-        self.assertEqual(out_lines[1], u"123456789 " * 40 + u"123456789")
-        self.assertEqual(out_lines[2], u"123456789 " * 8 + u"123456789")
+        self.assertEqual(out_lines[1], "123456789 " * 40 + "123456789")
+        self.assertEqual(out_lines[2], "123456789 " * 8 + "123456789")
         self.assertEqual(out_lines[3], in_lines[2])
 
     def test_force_line_max_length_in_p(self):
         """it should not have a line more than 400 chars"""
 
-        content = u"""
+        content = """
             <html>
             <body>
             </body>
@@ -1139,9 +1237,9 @@ class MaxLineLengthTest(BaseTestCase):
             <p>Paragraph 2</p>
             <p>{0}</p>
             </html>
-        """.format(u"123456789 " * 50)
+        """.format("123456789 " * 50)
 
-        expected_content = u"""
+        expected_content = """
             <html>
             <body>
             </body>
@@ -1153,8 +1251,8 @@ class MaxLineLengthTest(BaseTestCase):
             <p>Paragraph 2</p>\n<p>{0}\n{1}</p>
             </html>
         """.format(
-            u"123456789 " * 39 + u"123456789",
-            u"123456789 " * 10
+            "123456789 " * 39 + "123456789",
+            "123456789 " * 10
         )
 
         result = force_line_max_length(content, max_length_per_line=400, dont_cut_in_quotes=False)
@@ -1163,7 +1261,7 @@ class MaxLineLengthTest(BaseTestCase):
     def test_force_line_max_length_a1(self):
         """it should not have a line more than 400 chars"""
 
-        content = u"""
+        content = """
             <html>
             <body>
             </body>
@@ -1175,9 +1273,9 @@ class MaxLineLengthTest(BaseTestCase):
             <p>Paragraph 2</p>
             <p>{0}<a href="{1}">This is a link</a></p>
             </html>
-        """.format(u"123456789 " * 36, u"this-is-a-link-dont-cut-in-quotes")
+        """.format("123456789 " * 36, "this-is-a-link-dont-cut-in-quotes")
 
-        expected_content = u"""
+        expected_content = """
             <html>
             <body>
             </body>
@@ -1188,7 +1286,7 @@ class MaxLineLengthTest(BaseTestCase):
             </p>
             <p>Paragraph 2</p>\n<p>{0}<a href="{1}">This\nis a link</a></p>
             </html>
-        """.format(u"123456789 " * 36, u"this-is-a-link-dont-cut-in-quotes")
+        """.format("123456789 " * 36, "this-is-a-link-dont-cut-in-quotes")
 
         result = force_line_max_length(content,  max_length_per_line=400, dont_cut_in_quotes=False)
 
@@ -1197,7 +1295,7 @@ class MaxLineLengthTest(BaseTestCase):
     def test_force_line_max_length_a2(self):
         """it should not have a line more than 400 chars"""
 
-        content = u"""
+        content = """
             <html>
             <body>
             </body>
@@ -1209,9 +1307,9 @@ class MaxLineLengthTest(BaseTestCase):
             <p>Paragraph 2</p>
             <p>{0}<a class="abcdefgh ijklmnop qrstuvwxyz" href="{1}">This is a link</a></p>
             </html>
-        """.format(u"123456789 " * 38, u"this-is-a-link-dont-cut-in-quotes")
+        """.format("123456789 " * 38, "this-is-a-link-dont-cut-in-quotes")
 
-        expected_content = u"""
+        expected_content = """
             <html>
             <body>
             </body>
@@ -1222,7 +1320,7 @@ class MaxLineLengthTest(BaseTestCase):
             </p>
             <p>Paragraph 2</p>\n<p>{0}<a class="abcdefgh ijklmnop qrstuvwxyz"\nhref="{1}">This is a link</a></p>
             </html>
-        """.format(u"123456789 " * 38, u"this-is-a-link-dont-cut-in-quotes")
+        """.format("123456789 " * 38, "this-is-a-link-dont-cut-in-quotes")
 
         result = force_line_max_length(content, max_length_per_line=400, dont_cut_in_quotes=False)
         self.assertEqual(expected_content, result)
