@@ -2,19 +2,19 @@
 """test subscription"""
 
 import os
-from bs4 import BeautifulSoup
 from datetime import date
 
 from django.conf import settings
 from django.contrib.sites.models import Site
 from django.core import mail
-from django.core.urlresolvers import reverse
-from django.test import TestCase
 from django.test.utils import override_settings
+from django.urls import reverse
 from django.utils.translation import activate
 
+from coop_cms.tests import BeautifulSoup
 from model_mommy import mommy
 
+from balafon.unit_tests import TestCase
 from balafon.Crm import models
 from balafon.Crm.signals import new_subscription
 
@@ -23,13 +23,17 @@ from balafon.Crm.signals import new_subscription
 class SubscribeTest(TestCase):
     """Subscribe to newsletter"""
 
+    @staticmethod
+    def email_as_string(email):
+        return email.message().as_string()
+
     def setUp(self):
         """before each test"""
+        super(SubscribeTest, self).setUp()
         self._lang = settings.LANGUAGES[0][0]
         activate(self._lang)
 
-        if settings.BALAFON_USE_RECAPTCHA:
-            os.environ['RECAPTCHA_TESTING'] = 'True'
+        os.environ['RECAPTCHA_DISABLE'] = 'True'
 
         if not getattr(settings, 'BALAFON_ALLOW_SINGLE_CONTACT', True):
             settings.BALAFON_INDIVIDUAL_ENTITY_ID = models.EntityType.objects.create(name="particulier").id
@@ -39,9 +43,8 @@ class SubscribeTest(TestCase):
     def tearDown(self):
         """after each test"""
         activate(self._lang)
-
-        if settings.BALAFON_USE_RECAPTCHA:
-            os.environ['RECAPTCHA_TESTING'] = 'False'
+        os.environ['RECAPTCHA_DISABLE'] = 'False'
+        super(SubscribeTest, self).tearDown()
 
     def test_view_subscribe_newsletter(self):
         """view subscription page"""
@@ -105,7 +108,7 @@ class SubscribeTest(TestCase):
         verification_email = mail.outbox[1]
         self.assertEqual(verification_email.to, [contact.email])
         url = reverse('emailing_email_verification', args=[contact.uuid])
-        email_content = verification_email.message().as_string().decode('utf-8')
+        email_content = self.email_as_string(verification_email)
         self.assertTrue(email_content.find(url) > 0)
 
         notification_email = mail.outbox[0]
@@ -143,7 +146,7 @@ class SubscribeTest(TestCase):
         verification_email = mail.outbox[1]
         self.assertEqual(verification_email.to, [contact.email])
         url = reverse('emailing_email_verification', args=[contact.uuid])
-        email_content = verification_email.message().as_string().decode('utf-8')
+        email_content = self.email_as_string(verification_email)
         self.assertTrue(email_content.find(url) > 0)
 
         notification_email = mail.outbox[0]
@@ -187,7 +190,7 @@ class SubscribeTest(TestCase):
         verification_email = mail.outbox[1]
         self.assertEqual(verification_email.to, [contact.email])
         url = reverse('emailing_email_verification', args=[contact.uuid])
-        email_content = verification_email.message().as_string().decode('utf-8')
+        email_content = self.email_as_string(verification_email)
         self.assertTrue(email_content.find(url) > 0)
 
         notification_email = mail.outbox[0]
@@ -227,7 +230,7 @@ class SubscribeTest(TestCase):
         verification_email = mail.outbox[1]
         self.assertEqual(verification_email.to, [contact.email])
         url = reverse('emailing_email_verification', args=[contact.uuid])
-        email_content = verification_email.message().as_string().decode('utf-8')
+        email_content = self.email_as_string(verification_email)
         self.assertTrue(email_content.find(url) > 0)
 
         notification_email = mail.outbox[0]
@@ -275,12 +278,12 @@ class SubscribeTest(TestCase):
         verification_email = mail.outbox[1]
         self.assertEqual(verification_email.to, [new_contact.email])
         url = reverse('emailing_email_verification', args=[new_contact.uuid])
-        email_content = verification_email.message().as_string().decode('utf-8')
+        email_content = self.email_as_string(verification_email)
         self.assertTrue(email_content.find(url) > 0)
 
         notification_email = mail.outbox[0]
         self.assertEqual(notification_email.to, [settings.BALAFON_NOTIFICATION_EMAIL])
-        email_content = notification_email.message().as_string().decode('utf-8')
+        email_content = self.email_as_string(notification_email)
         self.assertTrue(email_content.find(new_contact.fullname) > 0)
         self.assertTrue(email_content.find(new_contact.get_absolute_url()) > 0)
         self.assertTrue(email_content.find(existing_contact.get_absolute_url()) > 0)
@@ -346,12 +349,12 @@ class SubscribeTest(TestCase):
         verification_email = mail.outbox[1]
         self.assertEqual(verification_email.to, [new_contact.email])
         url = reverse('emailing_email_verification', args=[new_contact.uuid])
-        email_content = verification_email.message().as_string().decode('utf-8')
+        email_content = self.email_as_string(verification_email)
         self.assertTrue(email_content.find(url) > 0)
 
         notification_email = mail.outbox[0]
         self.assertEqual(notification_email.to, [settings.BALAFON_NOTIFICATION_EMAIL])
-        email_content = notification_email.message().as_string().decode('utf-8')
+        email_content = self.email_as_string(notification_email)
         self.assertTrue(email_content.find(new_contact.fullname) > 0)
         self.assertTrue(email_content.find(new_contact.get_absolute_url()) > 0)
         for existing_contact in (existing_contact1, existing_contact2, existing_contact3, existing_contact4):
@@ -396,7 +399,7 @@ class SubscribeTest(TestCase):
         verification_email = mail.outbox[1]
         self.assertEqual(verification_email.to, [contact.email])
         url = reverse('emailing_email_verification', args=[contact.uuid])
-        email_content = verification_email.message().as_string().decode('utf-8')
+        email_content = self.email_as_string(verification_email)
         self.assertTrue(email_content.find(url) > 0)
 
         notification_email = mail.outbox[0]
@@ -436,19 +439,7 @@ class SubscribeTest(TestCase):
 
     def _patch_with_captcha(self, url, data):
         """path form data with captcha"""
-        if settings.BALAFON_USE_RECAPTCHA:
-            data.update({'recaptcha_response_field': 'PASSED'})
-
-        else:
-            from captcha.models import CaptchaStore
-            self.failUnlessEqual(CaptchaStore.objects.count(), 0)
-            self.client.get(url)
-            self.failUnlessEqual(CaptchaStore.objects.count(), 1)
-            captcha = CaptchaStore.objects.all()[0]
-            data.update({
-                'captcha_0': captcha.hashkey,
-                'captcha_1': captcha.response
-            })
+        pass
 
     def test_view_subscribe_with_types(self):
         """view subscribe page"""
@@ -499,7 +490,7 @@ class SubscribeTest(TestCase):
         self.assertEqual('', soup.select("#id_favorite_language")[0].get("value", ""))
 
     def test_accept_newsletter_not_in_site(self):
-        """if subscribtion is not set on the site, it should not be possible to subscribe"""
+        """if subscription is not set on the site, it should not be possible to subscribe"""
         newsletter_subscription = mommy.make(models.SubscriptionType, name="newsletter", site=None)
 
         url = reverse("emailing_subscribe_newsletter")
@@ -520,6 +511,42 @@ class SubscribeTest(TestCase):
         self.assertEqual(models.Subscription.objects.count(), 0)
         self.assertEqual(len(mail.outbox), 0)
 
+    def test_accept_newsletter_allowed_site(self):
+        """if subscription is not set on the site, it should not be possible to subscribe"""
+        site1 = Site.objects.get_current()
+        site2 = mommy.make(Site, domain="balafon.net")
+        newsletter_subscription = mommy.make(models.SubscriptionType, name="newsletter", site=site2)
+        newsletter_subscription.allowed_on_sites.add(site1)
+        newsletter_subscription.save()
+
+        url = reverse("emailing_subscribe_newsletter")
+
+        data = {
+            'entity_type': 0,
+            'lastname': 'Dupond',
+            'firstname': 'Pierre',
+            'email': 'pdupond@apidev.fr',
+            'subscription_types': [newsletter_subscription.id],
+        }
+
+        self._patch_with_captcha(url, data)
+
+        response = self.client.post(url, data=data, follow=False)
+        self.assertEqual(302, response.status_code)
+        self.assertEqual(models.Contact.objects.count(), 1)
+        contact = models.Contact.objects.all()[0]
+        self.assertEqual(models.Subscription.objects.count(), 1)
+        self.assertEqual(len(mail.outbox), 2)
+
+        verification_email = mail.outbox[1]
+        self.assertEqual(verification_email.to, [contact.email])
+        url = reverse('emailing_email_verification', args=[contact.uuid])
+        email_content = self.email_as_string(verification_email)
+        self.assertTrue(email_content.find(url) > 0)
+
+        notification_email = mail.outbox[0]
+        self.assertEqual(notification_email.to, [settings.BALAFON_NOTIFICATION_EMAIL])
+
     def test_accept_newsletter(self, accept_newsletter=True, accept_3rdparty=True):
         """subscribe and accept some subscribtions"""
         site1 = Site.objects.get_current()
@@ -582,7 +609,7 @@ class SubscribeTest(TestCase):
         verification_email = mail.outbox[1]
         self.assertEqual(verification_email.to, [contact.email])
         url = reverse('emailing_email_verification', args=[contact.uuid])
-        email_content = verification_email.message().as_string().decode('utf-8')
+        email_content = self.email_as_string(verification_email)
         self.assertTrue(email_content.find(url) > 0)
 
         notification_email = mail.outbox[0]
@@ -628,6 +655,35 @@ class SubscribeTest(TestCase):
         subscription2 = models.Subscription.objects.get(id=subscription2.id)
         self.assertEqual(subscription2.accept_subscription, True)
 
+    def test_verify_email_and_redirect_to(self):
+        """test email verification"""
+        self.client.logout()
+
+        site1 = Site.objects.get_current()
+
+        newsletter_subscription = mommy.make(models.SubscriptionType, name="newsletter", site=site1)
+        third_party_subscription = mommy.make(models.SubscriptionType, name="3rd_party", site=site1)
+        contact = mommy.make(models.Contact, email='toto@apidev.fr', email_verified=False)
+
+        subscription1 = mommy.make(
+            models.Subscription, contact=contact, subscription_type=newsletter_subscription, accept_subscription=True
+        )
+        subscription2 = mommy.make(
+            models.Subscription, contact=contact, subscription_type=third_party_subscription, accept_subscription=True
+        )
+
+        url = reverse('emailing_email_verification', args=[contact.uuid]) + '?redirect_to=https://test.fr/'
+        response = self.client.get(url)
+        self.assertRedirects(response, 'https://test.fr/', fetch_redirect_response=False)
+
+        contact = models.Contact.objects.get(id=contact.id)
+        self.assertEqual(contact.email_verified, True)
+
+        subscription1 = models.Subscription.objects.get(id=subscription1.id)
+        self.assertEqual(subscription1.accept_subscription, True)
+        subscription2 = models.Subscription.objects.get(id=subscription2.id)
+        self.assertEqual(subscription2.accept_subscription, True)
+
     def test_verify_email_no_newsletter(self):
         """test email verification ofr some who refuse subscriptions"""
         self.client.logout()
@@ -713,7 +769,7 @@ class SubscribeTest(TestCase):
 
         # Check the callbacks have been called
         self.assertEqual(models.Group.objects.count(), 1)
-        group = models.Group.objects.get(name=u'newsletter')
+        group = models.Group.objects.get(name='newsletter')
         self.assertEqual(group.entities.count(), 0)
         self.assertEqual(list(group.contacts.all()), [contact])
 
