@@ -4,20 +4,14 @@
 import datetime
 import os.path
 
-from django import VERSION as DJANGO_VERSION
 from django.conf import settings
 from django.contrib.auth.decorators import user_passes_test
 from django.contrib import messages
-if DJANGO_VERSION >= (1, 9, 0):
-    from wsgiref.util import FileWrapper
-else:
-    from django.core.servers.basehttp import FileWrapper
-from django.core.urlresolvers import reverse
 from django.views.generic.edit import UpdateView
 from django.http import HttpResponse, HttpResponseRedirect, Http404
 from django.shortcuts import render, get_object_or_404
-from django.template import RequestContext, Context
 from django.template.loader import get_template
+from django.urls import reverse
 from django.utils.decorators import method_decorator
 from django.utils.translation import ugettext as _
 from django.views.generic.base import View, TemplateView
@@ -25,6 +19,7 @@ from django.views.generic.base import View, TemplateView
 from colorbox.decorators import popup_redirect
 from coop_cms.models import Newsletter
 from coop_cms.utils import redirect_to_language, paginate
+from wsgiref.util import FileWrapper
 
 from balafon.permissions import can_access
 from balafon.utils import logger
@@ -60,7 +55,7 @@ def delete_emailing(request, emailing_id):
     """delete an emailing"""
 
     emailing = get_object_or_404(models.Emailing, id=emailing_id)
-    
+
     if request.method == 'POST':
         form = ConfirmForm(request.POST)
         if form.is_valid():
@@ -75,7 +70,7 @@ def delete_emailing(request, emailing_id):
         'balafon/confirmation_dialog.html',
         {
             'form': form,
-            'message': _(u"Are you sure to delete '{0.newsletter.subject}' {1}?").format(
+            'message': _("Are you sure to delete '{0.newsletter.subject}' {1}?").format(
                 emailing, emailing.get_status_display()),
             'action_url': reverse("emailing_delete", args=[emailing_id]),
         }
@@ -109,27 +104,27 @@ def new_newsletter(request):
                 source_url = form.cleaned_data["source_url"]
                 go_to_edit = False
                 if not content:
-                    content = _(u"Enter the text of your newsletter here")
+                    content = _("Enter the text of your newsletter here")
                     go_to_edit = True
-                
+
                 newsletter = Newsletter.objects.create(
                     subject=subject, template=template, content=content, source_url=source_url
                 )
-                
+
                 if go_to_edit:
                     return HttpResponseRedirect(newsletter.get_edit_url())
                 else:
                     return HttpResponseRedirect(newsletter.get_absolute_url())
         else:
             form = forms.NewNewsletterForm()
-    
+
         return render(
             request,
             'Emailing/new_newsletter.html',
             {'form': form}
         )
     except Exception as msg:
-        print "#ERR", msg
+        print("#ERR", msg)
         raise
 
 
@@ -145,19 +140,19 @@ def confirm_send_mail(request, emailing_id):
                 emailing = form.save()
                 emailing.status = models.Emailing.STATUS_SCHEDULED
                 emailing.save()
-                messages.add_message(request, messages.SUCCESS, _(u"The sending has been scheduled"))
+                messages.add_message(request, messages.SUCCESS, _("The sending has been scheduled"))
             else:
-                messages.add_message(request, messages.ERROR, _(u"The sending can't be scheduled"))
+                messages.add_message(request, messages.ERROR, _("The sending can't be scheduled"))
             return HttpResponseRedirect(reverse('emailing_newsletter_list'))
     else:
         form = forms.NewsletterSchedulingForm(instance=emailing)
-    
+
     return render(
         request,
         'Emailing/confirm_send_mail.html',
         {
             'form': form,
-            'message': _(u'Is this newsletter ready to be sent?'),
+            'message': _('Is this newsletter ready to be sent?'),
             'action_url': reverse("emailing_confirm_send_mail", args=[emailing_id]),
         }
     )
@@ -178,19 +173,19 @@ def cancel_send_mail(request, emailing_id):
                     emailing.status = models.Emailing.STATUS_EDITING
                     emailing.scheduling_dt = None
                     emailing.save()
-                    messages.add_message(request, messages.SUCCESS, _(u"The sending has been cancelled"))
+                    messages.add_message(request, messages.SUCCESS, _("The sending has been cancelled"))
                 else:
-                    messages.add_message(request, messages.ERROR, _(u"The sending can't be cancelled"))
+                    messages.add_message(request, messages.ERROR, _("The sending can't be cancelled"))
             return HttpResponseRedirect(reverse('emailing_newsletter_list'))
     else:
         form = ConfirmForm()
-        
+
     return render(
         request,
         'balafon/confirmation_dialog.html',
         {
             'form': form,
-            'message': _(u'Cancel the sending?'),
+            'message': _('Cancel the sending?'),
             'action_url': reverse("emailing_cancel_send_mail", args=[emailing_id]),
         }
     )
@@ -203,9 +198,9 @@ def view_link(request, link_uuid, contact_uuid):
     try:
         contact = Contact.objects.get(uuid=contact_uuid)
         link.visitors.add(contact)
-        
-        #create action
-        link_action = ActionType.objects.get_or_create(name=_(u'Link'))[0]
+
+        # create action
+        link_action = ActionType.objects.get_or_create(name=_('Link'))[0]
         action = Action.objects.create(
             subject=link.url, planned_date=now_rounded(),
             type=link_action, detail='', done=True, display_on_board=False,
@@ -213,7 +208,7 @@ def view_link(request, link_uuid, contact_uuid):
         )
         action.contacts.add(contact)
         action.save()
-        
+
     except Contact.DoesNotExist:
         pass
 
@@ -229,8 +224,8 @@ def unregister_contact(request, emailing_id, contact_uuid):
         my_company = emailing.subscription_type.name
     except models.Emailing.DoesNotExist:
         emailing = None
-        my_company = settings.BALAFON_MY_COMPANY
-    
+        my_company = getattr(settings, 'BALAFON_MY_COMPANY', '')
+
     if request.method == "POST":
         if 'unregister' in request.POST:
             form = forms.UnregisterForm(request.POST)
@@ -243,9 +238,9 @@ def unregister_contact(request, emailing_id, contact_uuid):
                     subscription.unsubscription_date = datetime.datetime.now()
                     subscription.save()
 
-                    emailing_action = ActionType.objects.get_or_create(name=_(u'Unregister'))[0]
+                    emailing_action = ActionType.objects.get_or_create(name=_('Unregister'))[0]
                     action = Action.objects.create(
-                        subject=_(u'{0} has unregister').format(contact),
+                        subject=_('{0} has unregister').format(contact),
                         planned_date=now_rounded(), type=emailing_action, detail=form.cleaned_data['reason'],
                         done=True, display_on_board=False, done_date=now_rounded()
                     )
@@ -267,8 +262,8 @@ def unregister_contact(request, emailing_id, contact_uuid):
                     }
                 )
             else:
-                pass #not valid : display with errors
-        
+                pass  # not valid : display with errors
+
         else:
             return render(
                 request,
@@ -298,7 +293,7 @@ def view_emailing_online(request, emailing_id, contact_uuid):
     """view an emailing online"""
     contact = get_object_or_404(Contact, uuid=contact_uuid)
     emailing = get_object_or_404(models.Emailing, id=emailing_id)
-    context = Context(get_emailing_context(emailing, contact))
+    context = get_emailing_context(emailing, contact)
     the_template = get_template(emailing.newsletter.get_template_name())
     html_text = the_template.render(context)
     html_text = patch_emailing_html(html_text, emailing, contact)
@@ -317,11 +312,10 @@ def subscribe_done(request, contact_uuid):
     subscription_type_names = [
         subscription.subscription_type.name for subscription in contact.subscription_set.all()
     ]
-    if subscription_type_names:
-        my_company = u', '.join(subscription_type_names)
-    else:
-        my_company = settings.BALAFON_MY_COMPANY
-    
+    my_company = ', '.join(subscription_type_names)
+    if not my_company:
+        my_company = getattr(settings, 'BALAFON_MY_COMPANY', '')
+
     return render(
         request,
         'Emailing/public/subscribe_done.html',
@@ -337,10 +331,11 @@ def subscribe_error(request, contact_uuid):
     contact = get_object_or_404(Contact, uuid=contact_uuid)
     subscription_type_names = [
         subscription.subscription_type.name for subscription in contact.subscription_set.all()
-        ]
-    if subscription_type_names:
-        my_company = u', '.join(subscription_type_names)
-    
+    ]
+    my_company = ', '.join(subscription_type_names)
+    if not my_company:
+        my_company = getattr(settings, 'BALAFON_MY_COMPANY', '')
+
     return render(
         request,
         'Emailing/public/subscribe_error.html',
@@ -356,36 +351,40 @@ def email_verification(request, contact_uuid):
     contact = get_object_or_404(Contact, uuid=contact_uuid)
     subscription_type_names = [
         subscription.subscription_type.name for subscription in contact.subscription_set.all()
-        ]
-    if subscription_type_names:
-        my_company = u', '.join(subscription_type_names)
-    else:
-        my_company = settings.BALAFON_MY_COMPANY
-    
+    ]
+    redirect_to = request.GET.get('redirect_to', None)
+
+    my_company = ', '.join(subscription_type_names)
+    if not my_company:
+        my_company = getattr(settings, 'BALAFON_MY_COMPANY', '')
+
     contact.email_verified = True
     contact.save()
-    
-    return render(
-        request,
-        'Emailing/public/verification_done.html',
-        {
-            'contact': contact,
-            'my_company': my_company,
-        }
-    )
+
+    if redirect_to:
+        return HttpResponseRedirect(redirect_to)
+    else:
+        return render(
+            request,
+            'Emailing/public/verification_done.html',
+            {
+                'contact': contact,
+                'my_company': my_company,
+            }
+        )
 
 
 def email_tracking(request, emailing_id, contact_uuid):
     """handle download of email opening tracking image"""
     emailing = get_object_or_404(models.Emailing, id=emailing_id)
     contact = get_object_or_404(Contact, uuid=contact_uuid)
-    
+
     emailing.opened_emails.add(contact)
     emailing.save()
-    
+
     dir_name = os.path.dirname(os.path.abspath(__file__))
     file_name = os.path.join(dir_name, "email-tracking.png")
-    response = HttpResponse(FileWrapper(open(file_name, 'r')), content_type='image/png')
+    response = HttpResponse(FileWrapper(open(file_name, 'rb')), content_type='image/png')
     response['Content-Length'] = os.path.getsize(file_name)
     return response
 
@@ -393,11 +392,11 @@ def email_tracking(request, emailing_id, contact_uuid):
 class SubscribeView(View):
     """Subscribe to emailing"""
     template_name = 'Emailing/public/subscribe.html'
-        
+
     def get_form_class(self):
         """get form: can be customized"""
         return get_subscription_form() or forms.SubscribeForm
-        
+
     def get_template(self):
         """get template"""
         return self.template_name
@@ -414,7 +413,7 @@ class SubscribeView(View):
     def get_success_url(self, contact):
         """where to redirect when form is valid"""
         return reverse('emailing_subscribe_done', args=[contact.uuid])
-    
+
     def get_error_url(self, contact):
         """where to redirect on error"""
         return reverse('emailing_subscribe_error', args=[contact.uuid])
@@ -465,20 +464,20 @@ class SubscribeView(View):
             except EmailSendError:
                 except_text = 'send_verification_email'
                 logger.exception(except_text)
-                
+
                 # create action
-                detail = _(u"An error occurred while verifying the email address of this contact.")
-                fix_action = ActionType.objects.get_or_create(name=_(u'Balafon'))[0]
+                detail = _("An error occurred while verifying the email address of this contact.")
+                fix_action = ActionType.objects.get_or_create(name=_('Balafon'))[0]
                 action = Action.objects.create(
-                    subject=_(u"Need to verify the email address"), planned_date=now_rounded(),
+                    subject=_("Need to verify the email address"), planned_date=now_rounded(),
                     type=fix_action, detail=detail, display_on_board=True,
                 )
                 action.contacts.add(contact)
                 action.save()
-                
+
                 return HttpResponseRedirect(self.get_error_url(contact))
         else:
-            except_text = u'contact form : {0}'.format(form.errors)
+            except_text = 'contact form : {0}'.format(form.errors)
             logger.warning(except_text)
         return self._display_form(form)
 
@@ -493,7 +492,7 @@ class EmailSubscribeView(SubscribeView):
     def get_success_url(self, contact):
         """where to redirect when form is valid"""
         return reverse('emailing_subscribe_email_done')
-    
+
     def get_error_url(self, contact):
         """where to redirect on error"""
         return reverse('emailing_subscribe_email_error')
