@@ -2,17 +2,18 @@
 """search test package"""
 
 from django.contrib.auth.models import User
-from django.test import TestCase
 
 from coop_cms.utils import RequestManager
 
+from balafon.unit_tests import TestCase
+
 
 class BaseTestCase(TestCase):
     """Base class for search tests"""
 
     def setUp(self):
         """before"""
-        RequestManager().clean()
+        super(BaseTestCase, self).setUp()
         self.user = User.objects.create(username="toto")
         self.user.set_password("abc")
         self.user.is_staff = True
