@@ -1,5 +1,4 @@
 # -*- coding: utf-8 -*-
-from __future__ import unicode_literals
 
 from django.db import migrations, models
 
@@ -26,11 +25,11 @@ class Migration(migrations.Migration):
         migrations.AlterField(
             model_name='sale',
             name='action',
-            field=models.OneToOneField(to='Crm.Action', verbose_name='action'),
+            field=models.OneToOneField(to='Crm.Action', verbose_name='action', on_delete=models.CASCADE),
         ),
         migrations.AddField(
             model_name='sale',
             name='analysis_code',
-            field=models.ForeignKey(default=None, blank=True, to='Store.SaleAnalysisCode', null=True, verbose_name='analysis code'),
+            field=models.ForeignKey(default=None, blank=True, to='Store.SaleAnalysisCode', null=True, verbose_name='analysis code', on_delete=models.CASCADE),
         ),
     ]
