@@ -8,9 +8,9 @@ from unittest import skipIf
 
 from django.conf import settings
 from django.contrib.auth.models import User
-from django.core.urlresolvers import reverse
 from django.core import mail
 from django.test.utils import override_settings
+from django.urls import reverse
 from django.utils.translation import ugettext as _
 
 from model_mommy import mommy
@@ -24,7 +24,7 @@ from balafon.Store.settings import get_cart_type_name
 
 
 def custom_confirmation_email_subject(profile, action):
-    return u'Hello {0}'.format(u''.join([contact.firstname for contact in action.contacts.all()]))
+    return 'Hello {0}'.format(''.join([contact.firstname for contact in action.contacts.all()]))
 
 
 GLOBAL_COUNTER = 0
@@ -40,7 +40,7 @@ class BaseTestCase(APITestCase):
 
     def setUp(self):
         logging.disable(logging.CRITICAL)
-        self.user = User.objects.create(username="toto", is_active=True, is_staff=False)
+        self.user = User.objects.create(username="toto", is_active=True, is_staff=False, email='toto@toto.fr')
         self.user.set_password("abc")
         self.user.save()
         self.client = APIClient()
@@ -124,7 +124,7 @@ class CartTest(BaseTestCase):
         store_item2 = models.StoreItem.objects.get(id=store_item2.id)
         self.assertEqual(store_item2.stock_count, None)
 
-    @override_settings(BALAFON_CART_CONFIRMATION_EMAIL_SUBJECT=u"Hello")
+    @override_settings(BALAFON_CART_CONFIRMATION_EMAIL_SUBJECT="Hello")
     def test_post_cart_email_subject(self):
         """It should create a new sale and action"""
 
@@ -184,7 +184,7 @@ class CartTest(BaseTestCase):
 
         self.assertEqual(len(mail.outbox), 2)
         self.assertEqual(mail.outbox[0].to, [profile.contact.email])
-        self.assertEqual(mail.outbox[0].subject, u"Hello")
+        self.assertEqual(mail.outbox[0].subject, "Hello")
         self.assertEqual(mail.outbox[1].to, [settings.BALAFON_NOTIFICATION_EMAIL])
 
         store_item1 = models.StoreItem.objects.get(id=store_item1.id)
@@ -200,7 +200,7 @@ class CartTest(BaseTestCase):
         # Create contact for the user
         profile = create_profile_contact(self.user)
         contact = profile.contact
-        contact.firstname = u"Bob"
+        contact.firstname = "Bob"
         contact.save()
 
         store_item1 = mommy.make(models.StoreItem)
@@ -255,7 +255,7 @@ class CartTest(BaseTestCase):
 
         self.assertEqual(len(mail.outbox), 2)
         self.assertEqual(mail.outbox[0].to, [profile.contact.email])
-        self.assertEqual(mail.outbox[0].subject, u"Hello Bob")
+        self.assertEqual(mail.outbox[0].subject, "Hello Bob")
         self.assertEqual(mail.outbox[1].to, [settings.BALAFON_NOTIFICATION_EMAIL])
 
         store_item1 = models.StoreItem.objects.get(id=store_item1.id)
@@ -303,7 +303,7 @@ class CartTest(BaseTestCase):
         action = action_queryset[0]
 
         self.assertEqual(list(action.contacts.all()), [contact])
-        self.assertEqual(action.subject, _(u'Notes'))
+        self.assertEqual(action.subject, _('Notes'))
         self.assertEqual(action.detail, data['notes'])
         self.assertEqual(action.planned_date, data['purchase_datetime'])
 
@@ -361,7 +361,7 @@ class CartTest(BaseTestCase):
         action = action_queryset[0]
 
         self.assertEqual(list(action.contacts.all()), [contact])
-        self.assertEqual(action.subject, _(u'Notes'))
+        self.assertEqual(action.subject, _('Notes'))
         self.assertEqual(action.detail, data['notes'])
         self.assertEqual(action.planned_date, data['purchase_datetime'])
 
@@ -419,7 +419,7 @@ class CartTest(BaseTestCase):
         action = action_queryset[0]
 
         self.assertEqual(list(action.contacts.all()), [contact])
-        self.assertEqual(action.subject, _(u'Notes'))
+        self.assertEqual(action.subject, _('Notes'))
         self.assertEqual(action.detail, 'a\nB\nc')
         self.assertEqual(action.planned_date, data['purchase_datetime'])
 
