@@ -1,17 +1,16 @@
 # -*- coding: utf-8 -*-
 """unit testing"""
 
-from bs4 import BeautifulSoup
 from decimal import Decimal
 
-from django.contrib.sites.models import Site
-from django.core.urlresolvers import reverse
-from django.test import TestCase
-from django.utils.translation import ugettext as _
+from django.contrib.auth.models import User
+from django.urls import reverse
 
+from coop_cms.tests import BeautifulSoup
 from model_mommy import mommy
 
-from balafon.Crm.models import Action, ActionMenu, ActionType, Contact
+from balafon.unit_tests import TestCase
+from balafon.Crm.models import Action, ActionMenu, ActionType, Contact, ActionStatus, MailtoSettings
 from balafon.Store import models
 
 
@@ -21,7 +20,7 @@ class StockThresholdTest(TestCase):
     def test_above_threshold(self):
         """It should not show a warning sign"""
         item = mommy.make(models.StoreItem, stock_count=40, stock_threshold=30)
-        soup = BeautifulSoup(item.stock_threshold_alert(), "html.parser")
+        soup = BeautifulSoup(item.stock_threshold_alert())
         self.assertEqual(0, len(soup.select("img")))
 
     def test_threshold_not_defined(self):
@@ -37,7 +36,7 @@ class StockThresholdTest(TestCase):
     def test_below_threshold(self):
         """It should show a warning sign"""
         item = mommy.make(models.StoreItem, stock_count=20, stock_threshold=30)
-        soup = BeautifulSoup(item.stock_threshold_alert(), "html.parser")
+        soup = BeautifulSoup(item.stock_threshold_alert())
         self.assertEqual(1, len(soup.select("img")))
 
 
@@ -343,7 +342,7 @@ class CategoryNameTest(TestCase):
         sub_category1 = mommy.make(models.StoreItemCategory, name='Def', parent=category1)
 
         self.assertEqual(category1.get_path_name(), category1.name)
-        self.assertEqual(sub_category1.get_path_name(), u'{0} > {1}'.format(category1.name, sub_category1.name))
+        self.assertEqual(sub_category1.get_path_name(), '{0} > {1}'.format(category1.name, sub_category1.name))
 
 
 class PricePolicyTest(TestCase):
@@ -547,7 +546,7 @@ class SaleDiscountTest(TestCase):
         tag = mommy.make(models.StoreItemTag, name="vrac")
 
         discount = mommy.make(
-            models.Discount, name=u'-10% à partir de 2 Kg', rate=Decimal("10"), quantity=Decimal("2"), active=True
+            models.Discount, name='-10% à partir de 2 Kg', rate=Decimal("10"), quantity=Decimal("2"), active=True
         )
         discount.tags.add(tag)
         discount.save()
@@ -578,7 +577,7 @@ class SaleDiscountTest(TestCase):
         vat_rate = mommy.make(models.VatRate, rate=Decimal("20.0"))
 
         discount = mommy.make(
-            models.Discount, name=u'-10% à partir de 2 Kg', rate=Decimal("10"), quantity=Decimal("2"), active=True
+            models.Discount, name='-10% à partir de 2 Kg', rate=Decimal("10"), quantity=Decimal("2"), active=True
         )
         price_class = mommy.make(models.PriceClass)
         price_class.discounts.add(discount)
@@ -610,7 +609,7 @@ class SaleDiscountTest(TestCase):
         tag = mommy.make(models.StoreItemTag, name="vrac")
 
         discount = mommy.make(
-            models.Discount, name=u'-10% à partir de 2 Kg', rate=Decimal("10"), quantity=Decimal("2"), active=False
+            models.Discount, name='-10% à partir de 2 Kg', rate=Decimal("10"), quantity=Decimal("2"), active=False
         )
         discount.tags.add(tag)
         discount.save()
@@ -643,7 +642,7 @@ class SaleDiscountTest(TestCase):
         tag = mommy.make(models.StoreItemTag, name="vrac")
 
         discount = mommy.make(
-            models.Discount, name=u'-10% à partir de 2 Kg', rate=Decimal("10"), quantity=Decimal("2"), active=False
+            models.Discount, name='-10% à partir de 2 Kg', rate=Decimal("10"), quantity=Decimal("2"), active=False
         )
         discount.tags.add(tag)
         discount.save()
@@ -676,13 +675,13 @@ class SaleDiscountTest(TestCase):
         tag = mommy.make(models.StoreItemTag, name="vrac")
 
         discount = mommy.make(
-            models.Discount, name=u'-10% à partir de 2 Kg', rate=Decimal("10"), quantity=Decimal("2"), active=True
+            models.Discount, name='-10% à partir de 2 Kg', rate=Decimal("10"), quantity=Decimal("2"), active=True
         )
         discount.tags.add(tag)
         discount.save()
 
         discount2 = mommy.make(
-            models.Discount, name=u'-20% à partir de 2 Kg', rate=Decimal("20"), quantity=Decimal("3"), active=True
+            models.Discount, name='-20% à partir de 2 Kg', rate=Decimal("20"), quantity=Decimal("3"), active=True
         )
         discount2.tags.add(tag)
         discount2.save()
@@ -715,13 +714,13 @@ class SaleDiscountTest(TestCase):
         tag = mommy.make(models.StoreItemTag, name="vrac")
 
         discount = mommy.make(
-            models.Discount, name=u'-10% à partir de 2 Kg', rate=Decimal("10"), quantity=Decimal("2"), active=True
+            models.Discount, name='-10% à partir de 2 Kg', rate=Decimal("10"), quantity=Decimal("2"), active=True
         )
         discount.tags.add(tag)
         discount.save()
 
         discount2 = mommy.make(
-            models.Discount, name=u'-20% à partir de 3 Kg', rate=Decimal("20"), quantity=Decimal("3"), active=True
+            models.Discount, name='-20% à partir de 3 Kg', rate=Decimal("20"), quantity=Decimal("3"), active=True
         )
         discount2.tags.add(tag)
         discount2.save()
@@ -754,7 +753,7 @@ class SaleDiscountTest(TestCase):
         tag = mommy.make(models.StoreItemTag, name="vrac")
 
         discount = mommy.make(
-            models.Discount, name=u'-10% à partir de 3 Kg', rate=Decimal("10"), quantity=Decimal("3"), active=True
+            models.Discount, name='-10% à partir de 3 Kg', rate=Decimal("10"), quantity=Decimal("3"), active=True
         )
         discount.tags.add(tag)
         discount.save()
@@ -828,20 +827,20 @@ class StoreItemDiscountTest(TestCase):
         tag4 = mommy.make(models.StoreItemTag, name="super-promos")
 
         discount = mommy.make(
-            models.Discount, name=u'-50% à partir de 4 Kg', rate=Decimal("50"), quantity=Decimal("4"), active=True
+            models.Discount, name='-50% à partir de 4 Kg', rate=Decimal("50"), quantity=Decimal("4"), active=True
         )
         discount.tags.add(tag1)
         discount.tags.add(tag2)
         discount.save()
 
         discount2 = mommy.make(
-            models.Discount, name=u'-20% à partir de 3 Kg', rate=Decimal("20"), quantity=Decimal("3"), active=True
+            models.Discount, name='-20% à partir de 3 Kg', rate=Decimal("20"), quantity=Decimal("3"), active=True
         )
         discount2.tags.add(tag1)
         discount2.save()
 
         discount3 = mommy.make(
-            models.Discount, name=u'-80% à partir de 3 Kg', rate=Decimal("80"), quantity=Decimal("3"), active=True
+            models.Discount, name='-80% à partir de 3 Kg', rate=Decimal("80"), quantity=Decimal("3"), active=True
         )
 
         item = mommy.make(models.StoreItem)
@@ -855,7 +854,7 @@ class StoreItemDiscountTest(TestCase):
         """if several tags are associated"""
 
         discount = mommy.make(
-            models.Discount, name=u'-50% à partir de 4 Kg', rate=Decimal("50"), quantity=Decimal("4"), active=True
+            models.Discount, name='-50% à partir de 4 Kg', rate=Decimal("50"), quantity=Decimal("4"), active=True
         )
 
         price_class = mommy.make(models.PriceClass)
@@ -872,14 +871,14 @@ class StoreItemDiscountTest(TestCase):
         tag2 = mommy.make(models.StoreItemTag, name="promo")
 
         discount = mommy.make(
-            models.Discount, name=u'-20% à partir de 4 Kg', rate=Decimal("20"), quantity=Decimal("4"), active=True
+            models.Discount, name='-20% à partir de 4 Kg', rate=Decimal("20"), quantity=Decimal("4"), active=True
         )
         discount.tags.add(tag1)
         discount.tags.add(tag2)
         discount.save()
 
         discount2 = mommy.make(
-            models.Discount, name=u'-10% à partir de 2 Kg', rate=Decimal("10"), quantity=Decimal("2"), active=True
+            models.Discount, name='-10% à partir de 2 Kg', rate=Decimal("10"), quantity=Decimal("2"), active=True
         )
         discount2.tags.add(tag1)
         discount2.save()
@@ -901,13 +900,13 @@ class StoreItemDiscountTest(TestCase):
         tag1 = mommy.make(models.StoreItemTag, name="vrac")
 
         discount = mommy.make(
-            models.Discount, name=u'-20% à partir de 4 Kg', rate=Decimal("20"), quantity=Decimal("4"), active=True
+            models.Discount, name='-20% à partir de 4 Kg', rate=Decimal("20"), quantity=Decimal("4"), active=True
         )
         discount.tags.add(tag1)
         discount.save()
 
         discount2 = mommy.make(
-            models.Discount, name=u'-10% à partir de 2 Kg', rate=Decimal("10"), quantity=Decimal("2"), active=True
+            models.Discount, name='-10% à partir de 2 Kg', rate=Decimal("10"), quantity=Decimal("2"), active=True
         )
 
         price_class = mommy.make(models.PriceClass)
@@ -923,11 +922,11 @@ class StoreItemDiscountTest(TestCase):
     def test_view_price_class_several_discounts(self):
         """if several price classes are associated"""
         discount = mommy.make(
-            models.Discount, name=u'-20% à partir de 4 Kg', rate=Decimal("20"), quantity=Decimal("4"), active=True
+            models.Discount, name='-20% à partir de 4 Kg', rate=Decimal("20"), quantity=Decimal("4"), active=True
         )
 
         discount2 = mommy.make(
-            models.Discount, name=u'-10% à partir de 2 Kg', rate=Decimal("10"), quantity=Decimal("2"), active=True
+            models.Discount, name='-10% à partir de 2 Kg', rate=Decimal("10"), quantity=Decimal("2"), active=True
         )
 
         price_class = mommy.make(models.PriceClass)
@@ -941,11 +940,11 @@ class StoreItemDiscountTest(TestCase):
     def test_view_price_class_no_discounts(self):
         """if no price classes are associated"""
         discount = mommy.make(
-            models.Discount, name=u'-20% à partir de 4 Kg', rate=Decimal("20"), quantity=Decimal("4"), active=False
+            models.Discount, name='-20% à partir de 4 Kg', rate=Decimal("20"), quantity=Decimal("4"), active=False
         )
 
         mommy.make(
-            models.Discount, name=u'-10% à partir de 2 Kg', rate=Decimal("10"), quantity=Decimal("2"), active=True
+            models.Discount, name='-10% à partir de 2 Kg', rate=Decimal("10"), quantity=Decimal("2"), active=True
         )
 
         price_class = mommy.make(models.PriceClass)
@@ -963,20 +962,20 @@ class StoreItemDiscountTest(TestCase):
         tag3 = mommy.make(models.StoreItemTag, name="soldes")
 
         discount = mommy.make(
-            models.Discount, name=u'-50% à partir de 4 Kg', rate=Decimal("50"), quantity=Decimal("4"), active=False
+            models.Discount, name='-50% à partir de 4 Kg', rate=Decimal("50"), quantity=Decimal("4"), active=False
         )
         discount.tags.add(tag1)
         discount.tags.add(tag2)
         discount.save()
 
         discount2 = mommy.make(
-            models.Discount, name=u'-20% à partir de 3 Kg', rate=Decimal("20"), quantity=Decimal("3"), active=True
+            models.Discount, name='-20% à partir de 3 Kg', rate=Decimal("20"), quantity=Decimal("3"), active=True
         )
         discount2.tags.add(tag1)
         discount2.save()
 
         discount3 = mommy.make(
-            models.Discount, name=u'-80% à partir de 3 Kg', rate=Decimal("80"), quantity=Decimal("3"), active=True
+            models.Discount, name='-80% à partir de 3 Kg', rate=Decimal("80"), quantity=Decimal("3"), active=True
         )
 
         item = mommy.make(models.StoreItem)
@@ -993,7 +992,7 @@ class StoreItemDiscountTest(TestCase):
         tag2 = mommy.make(models.StoreItemTag, name="promo")
 
         discount = mommy.make(
-            models.Discount, name=u'-50% à partir de 4 Kg', rate=Decimal("50"), quantity=Decimal("4"), active=False
+            models.Discount, name='-50% à partir de 4 Kg', rate=Decimal("50"), quantity=Decimal("4"), active=False
         )
         discount.tags.add(tag1)
         discount.tags.add(tag2)
@@ -1005,14 +1004,22 @@ class StoreItemDiscountTest(TestCase):
 
 
 class MailtoActionTest(TestCase):
-    """mailtto"""
+    """mailto"""
 
-    def test_mailto_to_action_body(self):
+    def setUp(self):
+        """before each test"""
+        super(MailtoActionTest, self).setUp()
+        user = mommy.make(User, is_active=True, is_staff=True, is_superuser=False)
+        user.set_password('abc')
+        user.save()
+        self.client.login(username=user.username, password='abc')
+
+    def test_mailto_to_action_body_https(self):
 
         action_type = mommy.make(ActionType, mail_to_subject='', generate_uuid=True, name='Doc')
         mommy.make(models.StoreManagementActionType, action_type=action_type)
 
-        action = mommy.make(Action, type=action_type, subject=u'Test')
+        action = mommy.make(Action, type=action_type, subject='Test')
 
         contact = mommy.make(Contact, email='toto@toto.fr', lastname='Doe', firstname='John')
         action.contacts.add(contact)
@@ -1021,22 +1028,73 @@ class MailtoActionTest(TestCase):
         self.assertNotEqual(action.uuid, '')
         url = reverse('store_view_sales_document_public', args=[action.uuid])
 
-        body = _(u"Here is a link to your {0}: {1}{2}").format(
-            action_type.name,
-            "http://" + Site.objects.get_current().domain,
-            url
+        expected_body = "Here is a link to your document: https://example.com{0}".format(url).replace(' ', '%20')
+
+        mommy.make(
+            MailtoSettings,
+            body_template="Here is a link to your document: {{ doc_url }}",
+            action_type=action_type
         )
 
-        self.assertEqual(
-            action.mail_to,
-            u'mailto:"John Doe" <toto@toto.fr>?subject=Test&body={0}'.format(body)
+        url = reverse("crm_mailto_action", args=[action.id])
+        self.assertEqual(action.mail_to, url)
+        response = self.client.get(url, secure=True)
+        self.assertEqual(response.status_code, 302)
+        self.assertEqual(response['Location'], "mailto:toto@toto.fr?subject=Test&body={0}".format(expected_body))
+
+    def test_mailto_to_action_body_http(self):
+
+        action_type = mommy.make(ActionType, mail_to_subject='', generate_uuid=True, name='Doc')
+        mommy.make(models.StoreManagementActionType, action_type=action_type)
+
+        action = mommy.make(Action, type=action_type, subject='Test')
+
+        contact = mommy.make(Contact, email='toto@toto.fr', lastname='Doe', firstname='John')
+        action.contacts.add(contact)
+        action.save()
+
+        self.assertNotEqual(action.uuid, '')
+        url = reverse('store_view_sales_document_public', args=[action.uuid])
+
+        expected_body = "Here is a link to your document: http://example.com{0}".format(url).replace(' ', '%20')
+
+        mommy.make(
+            MailtoSettings,
+            body_template="Here is a link to your document: {{ doc_url }}",
+            action_type=action_type
         )
 
+        url = reverse("crm_mailto_action", args=[action.id])
+        self.assertEqual(action.mail_to, url)
+        response = self.client.get(url, secure=False)
+        self.assertEqual(response.status_code, 302)
+        self.assertEqual(response['Location'], "mailto:toto@toto.fr?subject=Test&body={0}".format(expected_body))
+
     def test_mailto_to_action_body_no_sale(self):
 
         action_type = mommy.make(ActionType, mail_to_subject='', generate_uuid=True, name='Doc')
 
-        action = mommy.make(Action, type=action_type, subject=u'Test')
+        action = mommy.make(Action, type=action_type, subject='Test')
+
+        contact = mommy.make(Contact, email='toto@toto.fr', lastname='Doe', firstname='John')
+        action.contacts.add(contact)
+        action.save()
+
+        mommy.make(MailtoSettings, body_template="", action_type=action_type)
+
+        self.assertNotEqual(action.uuid, '')
+
+        url = reverse("crm_mailto_action", args=[action.id])
+        self.assertEqual(action.mail_to, url)
+        response = self.client.get(url)
+        self.assertEqual(response.status_code, 302)
+        self.assertEqual(response['Location'], "mailto:toto@toto.fr?subject=Test&body=")
+
+    def test_mailto_to_action_body_no_settings(self):
+
+        action_type = mommy.make(ActionType, mail_to_subject='', generate_uuid=True, name='Doc')
+
+        action = mommy.make(Action, type=action_type, subject='Test')
 
         contact = mommy.make(Contact, email='toto@toto.fr', lastname='Doe', firstname='John')
         action.contacts.add(contact)
@@ -1044,17 +1102,18 @@ class MailtoActionTest(TestCase):
 
         self.assertNotEqual(action.uuid, '')
 
-        self.assertEqual(
-            action.mail_to,
-            u'mailto:"John Doe" <toto@toto.fr>?subject=Test&body='
-        )
+        url = reverse("crm_mailto_action", args=[action.id])
+        self.assertEqual(action.mail_to, url)
+        response = self.client.get(url)
+        self.assertEqual(response.status_code, 302)
+        self.assertEqual(response['Location'], "mailto:toto@toto.fr?subject=Test&body=")
 
     def test_mailto_to_action_body_no_uuid(self):
 
         action_type = mommy.make(ActionType, mail_to_subject='', generate_uuid=False, name='Doc')
         mommy.make(models.StoreManagementActionType, action_type=action_type)
 
-        action = mommy.make(Action, type=action_type, subject=u'Test')
+        action = mommy.make(Action, type=action_type, subject='Test')
 
         contact = mommy.make(Contact, email='toto@toto.fr', lastname='Doe', firstname='John')
         action.contacts.add(contact)
@@ -1062,17 +1121,18 @@ class MailtoActionTest(TestCase):
 
         self.assertEqual(action.uuid, '')
 
-        self.assertEqual(
-            action.mail_to,
-            u'mailto:"John Doe" <toto@toto.fr>?subject=Test&body='
-        )
+        url = reverse("crm_mailto_action", args=[action.id])
+        self.assertEqual(action.mail_to, url)
+        response = self.client.get(url)
+        self.assertEqual(response.status_code, 302)
+        self.assertEqual(response['Location'], "mailto:toto@toto.fr?subject=Test&body=")
 
     def test_mailto_to_action_subject(self):
 
         action_type = mommy.make(ActionType, mail_to_subject='Another subject', generate_uuid=True, name='Doc')
         mommy.make(models.StoreManagementActionType, action_type=action_type)
 
-        action = mommy.make(Action, type=action_type, subject=u'Test')
+        action = mommy.make(Action, type=action_type, subject='Test')
 
         contact = mommy.make(Contact, email='toto@toto.fr', lastname='Doe', firstname='John')
         action.contacts.add(contact)
@@ -1081,16 +1141,11 @@ class MailtoActionTest(TestCase):
         self.assertNotEqual(action.uuid, '')
         url = reverse('store_view_sales_document_public', args=[action.uuid])
 
-        body = _(u"Here is a link to your {0}: {1}{2}").format(
-            action_type.name,
-            "http://" + Site.objects.get_current().domain,
-            url
-        )
-
-        self.assertEqual(
-            action.mail_to,
-            u'mailto:"John Doe" <toto@toto.fr>?subject=Another subject&body={0}'.format(body)
-        )
+        url = reverse("crm_mailto_action", args=[action.id])
+        self.assertEqual(action.mail_to, url)
+        response = self.client.get(url)
+        self.assertEqual(response.status_code, 302)
+        self.assertEqual(response['Location'], "mailto:toto@toto.fr?subject=Another%20subject&body=")
 
 
 class SaleItemOrderTest(TestCase):
@@ -1130,7 +1185,7 @@ class SaleItemOrderTest(TestCase):
         self.assertEqual(1, item.order_index)
 
 
-class SaleReferenceText(TestCase):
+class SaleReferenceTest(TestCase):
     """It should increment the order correctly"""
 
     def test_empty(self):
@@ -1153,3 +1208,60 @@ class SaleReferenceText(TestCase):
 
         self.assertNotEqual(action.sale, None)
         self.assertEqual(action.sale.get_references_text(), references_text)
+
+
+class FreezeActionTest(TestCase):
+    """It should increment the order correctly"""
+
+    def test_not_frozen(self):
+        """It should be empty"""
+        status1 = mommy.make(ActionStatus)
+        status2 = mommy.make(ActionStatus)
+        status3 = mommy.make(ActionStatus)
+
+        store_action_type = mommy.make(models.StoreManagementActionType)
+        store_action_type.default_readonly_status = status3
+        store_action_type.readonly_status.add(status2, status3)
+        store_action_type.save()
+
+        action = mommy.make(Action, type=store_action_type.action_type, status=status1)
+        self.assertEqual(action.frozen, False)
+
+    def test_frozen(self):
+        """It should be empty"""
+        status1 = mommy.make(ActionStatus)
+        status2 = mommy.make(ActionStatus)
+        status3 = mommy.make(ActionStatus)
+
+        store_action_type = mommy.make(models.StoreManagementActionType)
+        store_action_type.default_readonly_status = status3
+        store_action_type.readonly_status.add(status2, status3)
+        store_action_type.save()
+
+        action = mommy.make(Action, type=store_action_type.action_type, status=status2)
+        self.assertEqual(action.frozen, True)
+
+    def test_refreeze(self):
+        """It should be empty"""
+        status1 = mommy.make(ActionStatus)
+        status2 = mommy.make(ActionStatus)
+        status3 = mommy.make(ActionStatus)
+
+        store_action_type = mommy.make(models.StoreManagementActionType)
+        store_action_type.default_readonly_status = status3
+        store_action_type.readonly_status.add(status2, status3)
+        store_action_type.save()
+
+        action = mommy.make(Action, type=store_action_type.action_type, status=status1)
+        self.assertEqual(action.frozen, False)
+
+        action.frozen = True
+        action.save()
+        self.assertEqual(action.frozen, True)
+        self.assertEqual(action.status, status1)
+
+        action.status = status2
+        action.save()
+        self.assertEqual(action.frozen, True)
+        self.assertEqual(action.status, status2)
+
