@@ -2,24 +2,21 @@
 """unit testing"""
 
 from datetime import datetime, date, timedelta
-import json
-import logging
 from unittest import skipIf
-from StringIO import StringIO
 import sys
 
 from django.conf import settings
 from django.contrib.auth.models import User, Group, AnonymousUser
 from django.contrib.contenttypes.models import ContentType
 from django.core import mail, management
-from django.core.urlresolvers import reverse
 from django.template import Template, Context
-from django.test import TestCase
+from django.urls import reverse
 from django.utils import timezone
 
-from coop_cms.utils import RequestManager
+from coop_cms.moves import StringIO
 from model_mommy import mommy
 
+from balafon.unit_tests import TestCase, response_as_json
 from balafon.utils import is_allowed_homepage
 from balafon.Crm import models
 from balafon.Emailing.models import Emailing
@@ -31,18 +28,13 @@ class BaseTestCase(TestCase):
 
     def setUp(self):
         """before each test"""
-        logging.disable(logging.CRITICAL)
-        RequestManager().clean()
+        super(BaseTestCase, self).setUp()
         self.user = User.objects.create(username="toto")
         self.user.set_password("abc")
         self.user.is_staff = True
         self.user.save()
         self._login()
 
-    def tearDown(self):
-        """after each test"""
-        logging.disable(logging.NOTSET)
-
     def _login(self):
         """login user"""
         return self.client.login(username="toto", password="abc")
@@ -73,7 +65,7 @@ class NotifyDueActionsTestCase(BaseTestCase):
         buf = StringIO()
         sysout = sys.stdout
         sys.stdout = buf
-        management.call_command('notify_due_actions', verbosity=verbosity, interactive=False, stdout=buf)
+        management.call_command('notify_due_actions', verbosity=verbosity, stdout=buf)
         buf.seek(0, 0)
         sys.stdout = sysout
         return buf.readlines()
@@ -292,24 +284,24 @@ class UpdateFavoriteTestCase(BaseTestCase):
         
     def test_render_template(self):
         """make sure render do not fail"""
-        for i in xrange(5):
+        for i in range(5):
             mommy.make(User)
         
         template = Template(self._template_content())
         
         dummy_request = DummyRequest()
         setattr(dummy_request, 'user', self.user)
-        context = {
+        context = Context({
             'users': User.objects.all(),
             'request': dummy_request,
-        }
+        })
         
-        template.render(Context(context))
+        template.render(context)
         
     def test_render_template_not_logged(self):
         """render for anonymous"""
         self.client.logout()
-        for i in xrange(5):
+        for i in range(5):
             mommy.make(User)
         
         template = Template(self._template_content())
@@ -351,7 +343,7 @@ class UpdateFavoriteTestCase(BaseTestCase):
         
         response = self.client.post(reverse('users_toggle_favorite'), data)
         self.assertEqual(200, response.status_code)
-        resp_data = json.loads(response.content)
+        resp_data = response_as_json(response)
         self.assertEqual(True, resp_data['success'])
         self.assertEqual(True, resp_data['status'])
         self.assertEqual(1, Favorite.objects.count())
@@ -380,7 +372,7 @@ class UpdateFavoriteTestCase(BaseTestCase):
         }
         response = self.client.post(reverse('users_toggle_favorite'), data)
         self.assertEqual(200, response.status_code)
-        resp_data = json.loads(response.content)
+        resp_data = response_as_json(response)
         self.assertEqual(True, resp_data['success'])
         self.assertEqual(False, resp_data['status'])
         self.assertEqual(0, Favorite.objects.count())
@@ -396,7 +388,7 @@ class UpdateFavoriteTestCase(BaseTestCase):
         }
         response = self.client.post(reverse('users_toggle_favorite'), data)
         self.assertEqual(200, response.status_code)
-        resp_data = json.loads(response.content)
+        resp_data = response_as_json(response)
         self.assertEqual(False, resp_data['success'])
         
     def test_post_wrong_ct(self):
@@ -410,7 +402,7 @@ class UpdateFavoriteTestCase(BaseTestCase):
         }
         response = self.client.post(reverse('users_toggle_favorite'), data)
         self.assertEqual(200, response.status_code)
-        resp_data = json.loads(response.content)
+        resp_data = response_as_json(response)
         self.assertEqual(False, resp_data['success'])
         
         
@@ -432,8 +424,8 @@ class ListFavoritesTestCase(BaseTestCase):
     def test_list(self):
         """view list"""
         logged_user = self.user
-        faved_users = [mommy.make(User, username='user-{0}'.format(i)) for i in xrange(5)]
-        not_faved_users = [mommy.make(User, username='NOT-{0}'.format(i)) for i in xrange(5)]
+        faved_users = [mommy.make(User, username='user-{0}'.format(i)) for i in range(5)]
+        not_faved_users = [mommy.make(User, username='NOT-{0}'.format(i)) for i in range(5)]
         
         user_ct = ContentType.objects.get_for_model(User)
         for user in faved_users:
@@ -450,8 +442,8 @@ class ListFavoritesTestCase(BaseTestCase):
         """do not see someone else list"""
         logged_user = self.user
         other_user = mommy.make(User)
-        faved_users = [mommy.make(User, username='user-{0}'.format(i)) for i in xrange(5)]
-        not_my_faved_users = [mommy.make(User, username='OTHER-{0}'.format(i)) for i in xrange(5)]
+        faved_users = [mommy.make(User, username='user-{0}'.format(i)) for i in range(5)]
+        not_my_faved_users = [mommy.make(User, username='OTHER-{0}'.format(i)) for i in range(5)]
         
         user_ct = ContentType.objects.get_for_model(User)
         
@@ -472,8 +464,8 @@ class ListFavoritesTestCase(BaseTestCase):
     def test_list_different_cts(self):
         """different contentypes in list"""
         logged_user = self.user
-        faved_users = [mommy.make(User, username='user-{0}'.format(i)) for i in xrange(5)]
-        faved_groups = [mommy.make(Group, name='group-{0}'.format(i)) for i in xrange(5)]
+        faved_users = [mommy.make(User, username='user-{0}'.format(i)) for i in range(5)]
+        faved_groups = [mommy.make(Group, name='group-{0}'.format(i)) for i in range(5)]
         
         user_ct = ContentType.objects.get_for_model(User)
         group_ct = ContentType.objects.get_for_model(Group)
@@ -548,7 +540,7 @@ class UserHomepageTestCase(BaseTestCase):
         response = self.client.post(url, data={'url': homepage_url})
         self.assertEqual(200, response.status_code)
 
-        response_data = json.loads(response.content)
+        response_data = response_as_json(response)
 
         self.assertEqual(response_data["ok"], False)
 
@@ -571,7 +563,7 @@ class UserHomepageTestCase(BaseTestCase):
         response = self.client.post(url, data={'url': homepage_url})
         self.assertEqual(200, response.status_code)
 
-        response_data = json.loads(response.content)
+        response_data = response_as_json(response)
 
         self.assertEqual(response_data["ok"], True)
 
@@ -630,7 +622,7 @@ class UserHomepageTestCase(BaseTestCase):
 
 
 @skipIf(
-    "balafon.Users.context_processors.user_config" not in settings.TEMPLATE_CONTEXT_PROCESSORS,
+    "balafon.Users.context_processors.user_config" not in settings.TEMPLATES[0]['OPTIONS']['context_processors'],
     "User context processor not set"
 )
 class CustomMenuTestCase(BaseTestCase):
@@ -814,3 +806,16 @@ class CustomMenuTestCase(BaseTestCase):
         self.assertNotContains(response, menu.label)
         self.assertNotContains(response, menu_item.label)
 
+    def test_view_custom_menu_css_class(self):
+        """It should display custom menu"""
+        menu = mommy.make(CustomMenu, label="MON MENU", position=CustomMenu.POSITION_MENU)
+        menu_item = mommy.make(
+            CustomMenuItem, parent=menu, label="MON ELEMENT", url='/test', attributes='class="custom-class"'
+        )
+
+        response = self.client.get(reverse("balafon_homepage"), follow=True)
+        self.assertEqual(200, response.status_code)
+
+        self.assertContains(response, menu.label)
+        self.assertContains(response, menu_item.label)
+        self.assertContains(response, 'class="custom-class dropdown-link"')
