@@ -3,8 +3,9 @@
 
 from django.contrib.auth.forms import AuthenticationForm, PasswordResetForm, PasswordChangeForm
 
+import floppyforms.__future__ as forms
+
 from coop_cms.bs_forms import BootstrapableMixin
-import floppyforms as forms
 
 
 class BsAuthenticationForm(BootstrapableMixin, AuthenticationForm):
